@@ -22,16 +22,18 @@
 
 /**
  * SECTION:element-alsasink
+ * @title: alsasink
  * @see_also: alsasrc
  *
  * This element renders audio samples using the ALSA audio API.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v uridecodebin uri=file:///path/to/audio.ogg ! audioconvert ! audioresample ! autoaudiosink
- * ]| Play an Ogg/Vorbis file and output audio via ALSA.
- * </refsect2>
+ * ]|
+ *
+ * Play an Ogg/Vorbis file and output audio via ALSA.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -166,8 +168,8 @@ gst_alsasink_class_init (GstAlsaSinkClass * klass)
       "Audio sink (ALSA)", "Sink/Audio",
       "Output to a sound card via ALSA", "Wim Taymans <wim@fluendo.com>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&alsasink_sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &alsasink_sink_factory);
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_alsasink_getcaps);
   gstbasesink_class->query = GST_DEBUG_FUNCPTR (gst_alsasink_query);
@@ -909,16 +911,8 @@ gst_alsasink_prepare (GstAudioSink * asink, GstAudioRingBufferSpec * spec)
   }
 
 #ifdef SND_CHMAP_API_VERSION
-  if (spec->type == GST_AUDIO_RING_BUFFER_FORMAT_TYPE_RAW && alsa->channels < 9) {
-    snd_pcm_chmap_t *chmap = snd_pcm_get_chmap (alsa->handle);
-    if (chmap && chmap->channels == alsa->channels) {
-      GstAudioChannelPosition pos[8];
-      if (alsa_chmap_to_channel_positions (chmap, pos))
-        gst_audio_ring_buffer_set_channel_positions (GST_AUDIO_BASE_SINK
-            (alsa)->ringbuffer, pos);
-    }
-    free (chmap);
-  }
+  alsa_detect_channels_mapping (GST_OBJECT (alsa), alsa->handle, spec,
+      alsa->channels, GST_AUDIO_BASE_SINK (alsa)->ringbuffer);
 #endif /* SND_CHMAP_API_VERSION */
 
   return TRUE;
@@ -1169,6 +1163,8 @@ gst_alsasink_payload (GstAudioBaseSink * sink, GstBuffer * buf)
 
     if (!gst_audio_iec61937_payload (iinfo.data, iinfo.size,
             oinfo.data, oinfo.size, &sink->ringbuffer->spec, G_BIG_ENDIAN)) {
+      gst_buffer_unmap (buf, &iinfo);
+      gst_buffer_unmap (out, &oinfo);
       gst_buffer_unref (out);
       return NULL;
     }
