@@ -172,7 +172,7 @@ gst_ogg_stream_granulepos_to_key_granule (GstOggStream * pad, gint64 granulepos)
   if (mappers[pad->map].granulepos_to_key_granule_func)
     return mappers[pad->map].granulepos_to_key_granule_func (pad, granulepos);
 
-  if (granulepos == -1 || granulepos == 0) {
+  if (granulepos == -1 || granulepos == 0 || pad->granuleshift == G_MAXUINT32) {
     return granulepos;
   }
 
@@ -289,7 +289,7 @@ gst_ogg_stream_update_stats (GstOggStream * pad, ogg_packet * packet)
   if (!mappers[pad->map].get_headers_func)
     return;
 
-  return mappers[pad->map].update_stats_func (pad, packet);
+  mappers[pad->map].update_stats_func (pad, packet);
 }
 
 /* some generic functions */
@@ -311,7 +311,7 @@ granulepos_to_granule_default (GstOggStream * pad, gint64 granulepos)
 {
   gint64 keyindex, keyoffset;
 
-  if (pad->granuleshift != 0) {
+  if (pad->granuleshift != 0 && pad->granuleshift != G_MAXUINT32) {
     keyindex = granulepos >> pad->granuleshift;
     keyoffset = granulepos - (keyindex << pad->granuleshift);
     return keyindex + keyoffset;
@@ -327,7 +327,7 @@ granule_to_granulepos_default (GstOggStream * pad, gint64 granule,
 {
   gint64 keyoffset;
 
-  if (pad->granuleshift != 0) {
+  if (pad->granuleshift != 0 && pad->granuleshift != G_MAXUINT32) {
     /* If we don't know where the previous keyframe is yet, assume it is
        at 0 or 1, depending on bitstream version. If nothing else, this
        avoids getting negative granpos back. */
@@ -392,7 +392,7 @@ tag_list_from_vorbiscomment_packet (ogg_packet * packet,
   }
 
   if (encoder) {
-    if (encoder[0])
+    if (encoder[0] && g_utf8_validate (encoder, -1, NULL))
       gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER, encoder,
           NULL);
     g_free (encoder);
@@ -478,7 +478,7 @@ granulepos_to_granule_theora (GstOggStream * pad, gint64 granulepos)
 {
   gint64 keyindex, keyoffset;
 
-  if (pad->granuleshift != 0) {
+  if (pad->granuleshift != 0 && pad->granuleshift != G_MAXUINT32) {
     keyindex = granulepos >> pad->granuleshift;
     keyoffset = granulepos - (keyindex << pad->granuleshift);
     if (pad->theora_has_zero_keyoffset) {
@@ -495,7 +495,7 @@ is_granulepos_keyframe_theora (GstOggStream * pad, gint64 granulepos)
 {
   gint64 frame_mask;
 
-  if (granulepos == (gint64) - 1)
+  if (granulepos == (gint64) - 1 || pad->granuleshift == G_MAXUINT32)
     return FALSE;
 
   frame_mask = (G_GUINT64_CONSTANT (1) << pad->granuleshift) - 1;
@@ -811,6 +811,9 @@ extract_tags_vp8 (GstOggStream * pad, ogg_packet * packet)
     tag_list_from_vorbiscomment_packet (packet,
         (const guint8 *) "OVP80\2 ", 7, &pad->taglist);
 
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new_empty ();
+
     gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
         GST_TAG_VIDEO_CODEC, "VP8", NULL);
   }
@@ -1047,6 +1050,9 @@ static gboolean
 is_header_fLaC (GstOggStream * pad, ogg_packet * packet)
 {
   if (pad->n_header_packets_seen == 1) {
+    if (packet->bytes < 17)
+      return FALSE;
+
     pad->granulerate_n = (packet->packet[14] << 12) |
         (packet->packet[15] << 4) | ((packet->packet[16] >> 4) & 0xf);
   }
@@ -1142,6 +1148,9 @@ extract_tags_flac (GstOggStream * pad, ogg_packet * packet)
     tag_list_from_vorbiscomment_packet (packet,
         packet->packet, 4, &pad->taglist);
 
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new_empty ();
+
     gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
         GST_TAG_AUDIO_CODEC, "FLAC", NULL);
   }
@@ -1156,6 +1165,11 @@ setup_fishead_mapper (GstOggStream * pad, ogg_packet * packet)
   gint64 prestime_n, prestime_d;
   gint64 basetime_n, basetime_d;
 
+  if (packet->bytes < 44) {
+    GST_DEBUG ("Not enough data for fishead header");
+    return FALSE;
+  }
+
   data = packet->packet;
 
   data += 8;                    /* header */
@@ -1564,7 +1578,7 @@ static gint64
 packet_duration_ogm (GstOggStream * pad, ogg_packet * packet)
 {
   const guint8 *data;
-  int samples;
+  gint64 samples;
   int offset;
   int n;
 
@@ -1572,7 +1586,7 @@ packet_duration_ogm (GstOggStream * pad, ogg_packet * packet)
   offset = 1 + (((data[0] & 0xc0) >> 6) | ((data[0] & 0x02) << 1));
 
   if (offset > packet->bytes) {
-    GST_ERROR ("buffer too small");
+    GST_WARNING ("buffer too small");
     return -1;
   }
 
@@ -2030,20 +2044,9 @@ granulepos_to_granule_opus (GstOggStream * pad, gint64 granulepos)
   if (granulepos == -1)
     return -1;
 
-  /* We must reject some particular cases for the first granpos */
-
   if (pad->first_granpos < 0 || granulepos < pad->first_granpos)
     pad->first_granpos = granulepos;
 
-  if (pad->first_granpos == granulepos) {
-    if (granulepos < -pad->granule_offset) {
-      GST_ERROR ("Invalid Opus stream: first granulepos (%" G_GINT64_FORMAT
-          ") less than preskip (%" G_GINT64_FORMAT ")", granulepos,
-          -pad->granule_offset);
-      return -1;
-    }
-  }
-
   return granulepos;
 }
 
@@ -2113,6 +2116,9 @@ extract_tags_opus (GstOggStream * pad, ogg_packet * packet)
     tag_list_from_vorbiscomment_packet (packet,
         (const guint8 *) "OpusTags", 8, &pad->taglist);
 
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new_empty ();
+
     gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
         GST_TAG_AUDIO_CODEC, "Opus", NULL);
   }
@@ -2184,7 +2190,7 @@ granulepos_to_granule_daala (GstOggStream * pad, gint64 granulepos)
 {
   gint64 keyindex, keyoffset;
 
-  if (pad->granuleshift != 0) {
+  if (pad->granuleshift != 0 && pad->granuleshift != G_MAXUINT32) {
     keyindex = granulepos >> pad->granuleshift;
     keyoffset = granulepos - (keyindex << pad->granuleshift);
     return keyindex + keyoffset;
@@ -2198,7 +2204,7 @@ is_granulepos_keyframe_daala (GstOggStream * pad, gint64 granulepos)
 {
   gint64 frame_mask;
 
-  if (granulepos == (gint64) - 1)
+  if (granulepos == (gint64) - 1 || pad->granuleshift == G_MAXUINT32)
     return FALSE;
 
   frame_mask = (G_GUINT64_CONSTANT (1) << pad->granuleshift) - 1;
@@ -2242,6 +2248,23 @@ extract_tags_daala (GstOggStream * pad, ogg_packet * packet)
 /* *INDENT-OFF* */
 /* indent hates our freedoms */
 const GstOggMap mappers[] = {
+  {
+    /* Empty mapper for uninitialized pads/streams */
+    NULL, 0, G_MAXINT32,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+  },
   {
     "\200theora", 7, 42,
     "video/x-theora",
