@@ -19,6 +19,7 @@
  */
 /**
  * SECTION:gstappsrc
+ * @title: GstAppSrc
  * @short_description: Easy way for applications to inject buffers into a
  *     pipeline
  * @see_also: #GstBaseSrc, appsink
@@ -94,17 +95,25 @@
 #endif
 
 #include <gst/gst.h>
-#include <gst/base/gstbasesrc.h>
+#include <gst/base/base.h>
 
 #include <string.h>
 
 #include "gstappsrc.h"
 
+typedef enum
+{
+  NOONE_WAITING,
+  STREAM_WAITING,               /* streaming thread is waiting for application thread */
+  APP_WAITING,                  /* application thread is waiting for streaming thread */
+} GstAppSrcWaitStatus;
+
 struct _GstAppSrcPrivate
 {
   GCond cond;
   GMutex mutex;
-  GQueue *queue;
+  GstQueueArray *queue;
+  GstAppSrcWaitStatus wait_status;
 
   GstCaps *last_caps;
   GstCaps *current_caps;
@@ -148,6 +157,7 @@ enum
   SIGNAL_PUSH_BUFFER,
   SIGNAL_END_OF_STREAM,
   SIGNAL_PUSH_SAMPLE,
+  SIGNAL_PUSH_BUFFER_LIST,
 
   LAST_SIGNAL
 };
@@ -190,26 +200,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-GType
-gst_app_stream_type_get_type (void)
-{
-  static volatile gsize stream_type_type = 0;
-  static const GEnumValue stream_type[] = {
-    {GST_APP_STREAM_TYPE_STREAM, "GST_APP_STREAM_TYPE_STREAM", "stream"},
-    {GST_APP_STREAM_TYPE_SEEKABLE, "GST_APP_STREAM_TYPE_SEEKABLE", "seekable"},
-    {GST_APP_STREAM_TYPE_RANDOM_ACCESS, "GST_APP_STREAM_TYPE_RANDOM_ACCESS",
-        "random-access"},
-    {0, NULL, NULL}
-  };
-
-  if (g_once_init_enter (&stream_type_type)) {
-    GType tmp = g_enum_register_static ("GstAppStreamType", stream_type);
-    g_once_init_leave (&stream_type_type, tmp);
-  }
-
-  return (GType) stream_type_type;
-}
-
 static void gst_app_src_uri_handler_init (gpointer g_iface,
     gpointer iface_data);
 
@@ -243,6 +233,8 @@ static gboolean gst_app_src_event (GstBaseSrc * src, GstEvent * event);
 
 static GstFlowReturn gst_app_src_push_buffer_action (GstAppSrc * appsrc,
     GstBuffer * buffer);
+static GstFlowReturn gst_app_src_push_buffer_list_action (GstAppSrc * appsrc,
+    GstBufferList * buffer_list);
 static GstFlowReturn gst_app_src_push_sample_action (GstAppSrc * appsrc,
     GstSample * sample);
 
@@ -484,6 +476,27 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           push_buffer), NULL, NULL, NULL,
       GST_TYPE_FLOW_RETURN, 1, GST_TYPE_BUFFER);
 
+   /**
+    * GstAppSrc::push-buffer-list:
+    * @appsrc: the appsrc
+    * @buffer_list: a buffer list to push
+    *
+    * Adds a buffer list to the queue of buffers and buffer lists that the
+    * appsrc element will push to its source pad. This function does not take
+    * ownership of the buffer list so the buffer list needs to be unreffed
+    * after calling this function.
+    *
+    * When the block property is TRUE, this function can block until free space
+    * becomes available in the queue.
+    *
+    * Since: 1.14
+    */
+  gst_app_src_signals[SIGNAL_PUSH_BUFFER_LIST] =
+      g_signal_new ("push-buffer-list", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstAppSrcClass,
+          push_buffer_list), NULL, NULL, NULL,
+      GST_TYPE_FLOW_RETURN, 1, GST_TYPE_BUFFER_LIST);
+
   /**
     * GstAppSrc::push-sample:
     * @appsrc: the appsrc
@@ -546,6 +559,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   basesrc_class->event = gst_app_src_event;
 
   klass->push_buffer = gst_app_src_push_buffer_action;
+  klass->push_buffer_list = gst_app_src_push_buffer_list_action;
   klass->push_sample = gst_app_src_push_sample_action;
   klass->end_of_stream = gst_app_src_end_of_stream;
 
@@ -562,7 +576,8 @@ gst_app_src_init (GstAppSrc * appsrc)
 
   g_mutex_init (&priv->mutex);
   g_cond_init (&priv->cond);
-  priv->queue = g_queue_new ();
+  priv->queue = gst_queue_array_new (16);
+  priv->wait_status = NOONE_WAITING;
 
   priv->size = DEFAULT_PROP_SIZE;
   priv->duration = DEFAULT_PROP_DURATION;
@@ -586,8 +601,8 @@ gst_app_src_flush_queued (GstAppSrc * src, gboolean retain_last_caps)
   GstAppSrcPrivate *priv = src->priv;
   GstCaps *requeue_caps = NULL;
 
-  while (!g_queue_is_empty (priv->queue)) {
-    obj = g_queue_pop_head (priv->queue);
+  while (!gst_queue_array_is_empty (priv->queue)) {
+    obj = gst_queue_array_pop_head (priv->queue);
     if (obj) {
       if (GST_IS_CAPS (obj) && retain_last_caps) {
         gst_caps_replace (&requeue_caps, GST_CAPS_CAST (obj));
@@ -597,7 +612,7 @@ gst_app_src_flush_queued (GstAppSrc * src, gboolean retain_last_caps)
   }
 
   if (requeue_caps) {
-    g_queue_push_tail (priv->queue, requeue_caps);
+    gst_queue_array_push_tail (priv->queue, requeue_caps);
   }
 
   priv->queued_bytes = 0;
@@ -641,7 +656,7 @@ gst_app_src_finalize (GObject * obj)
 
   g_mutex_clear (&priv->mutex);
   g_cond_clear (&priv->cond);
-  g_queue_free (priv->queue);
+  gst_queue_array_free (priv->queue);
 
   g_free (priv->uri);
 
@@ -758,7 +773,7 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
       break;
     case PROP_MIN_LATENCY:
     {
-      guint64 min;
+      guint64 min = 0;
 
       gst_app_src_get_latency (appsrc, &min, NULL);
       g_value_set_int64 (value, min);
@@ -766,7 +781,7 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
     }
     case PROP_MAX_LATENCY:
     {
-      guint64 max;
+      guint64 max = 0;
 
       gst_app_src_get_latency (appsrc, NULL, &max);
       g_value_set_int64 (value, max);
@@ -957,8 +972,12 @@ gst_app_src_query (GstBaseSrc * src, GstQuery * query)
         gst_query_set_duration (query, format, priv->size);
         res = TRUE;
       } else if (format == GST_FORMAT_TIME) {
-        gst_query_set_duration (query, format, priv->duration);
-        res = TRUE;
+        if (priv->duration != GST_CLOCK_TIME_NONE) {
+          gst_query_set_duration (query, format, priv->duration);
+          res = TRUE;
+        } else {
+          res = FALSE;
+        }
       } else {
         res = FALSE;
       }
@@ -1163,11 +1182,11 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
 
   while (TRUE) {
     /* return data as long as we have some */
-    if (!g_queue_is_empty (priv->queue)) {
+    if (!gst_queue_array_is_empty (priv->queue)) {
       guint buf_size;
-      GstMiniObject *obj = g_queue_pop_head (priv->queue);
+      GstMiniObject *obj = gst_queue_array_pop_head (priv->queue);
 
-      if (!GST_IS_BUFFER (obj)) {
+      if (GST_IS_CAPS (obj)) {
         GstCaps *next_caps = GST_CAPS (obj);
         gboolean caps_changed = TRUE;
 
@@ -1196,10 +1215,25 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
         continue;
       }
 
-      *buf = GST_BUFFER (obj);
-      buf_size = gst_buffer_get_size (*buf);
+      if (GST_IS_BUFFER (obj)) {
+        *buf = GST_BUFFER (obj);
+        buf_size = gst_buffer_get_size (*buf);
+        GST_LOG_OBJECT (appsrc, "have buffer %p of size %u", *buf, buf_size);
+      } else {
+        GstBufferList *buffer_list;
 
-      GST_DEBUG_OBJECT (appsrc, "we have buffer %p of size %u", *buf, buf_size);
+        g_assert (GST_IS_BUFFER_LIST (obj));
+
+        buffer_list = GST_BUFFER_LIST (obj);
+
+        buf_size = gst_buffer_list_calculate_size (buffer_list);
+
+        GST_LOG_OBJECT (appsrc, "have buffer list %p of size %u, %u buffers",
+            buffer_list, buf_size, gst_buffer_list_length (buffer_list));
+
+        gst_base_src_submit_buffer_list (bsrc, buffer_list);
+        *buf = NULL;
+      }
 
       priv->queued_bytes -= buf_size;
 
@@ -1208,7 +1242,8 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
         priv->offset += buf_size;
 
       /* signal that we removed an item */
-      g_cond_broadcast (&priv->cond);
+      if (priv->wait_status == APP_WAITING)
+        g_cond_broadcast (&priv->cond);
 
       /* see if we go lower than the empty-percent */
       if (priv->min_percent && priv->max_bytes) {
@@ -1231,7 +1266,7 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
        * signal) we can still be empty because the pushed buffer got flushed or
        * when the application pushes the requested buffer later, we support both
        * possibilities. */
-      if (!g_queue_is_empty (priv->queue))
+      if (!gst_queue_array_is_empty (priv->queue))
         continue;
 
       /* no buffer yet, maybe we are EOS, if not, block for more data. */
@@ -1242,7 +1277,9 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
       goto eos;
 
     /* nothing to return, wait a while for new data or flushing. */
+    priv->wait_status = STREAM_WAITING;
     g_cond_wait (&priv->cond, &priv->mutex);
+    priv->wait_status = NOONE_WAITING;
   }
   g_mutex_unlock (&priv->mutex);
   return ret;
@@ -1301,12 +1338,15 @@ gst_app_src_set_caps (GstAppSrc * appsrc, const GstCaps * caps)
 
   if (caps_changed) {
     GstCaps *new_caps;
+    gpointer t;
+
     new_caps = caps ? gst_caps_copy (caps) : NULL;
     GST_DEBUG_OBJECT (appsrc, "setting caps to %" GST_PTR_FORMAT, caps);
-    if (priv->queue->tail != NULL && GST_IS_CAPS (priv->queue->tail->data)) {
-      gst_caps_unref (g_queue_pop_tail (priv->queue));
+
+    while ((t = gst_queue_array_peek_tail (priv->queue)) && GST_IS_CAPS (t)) {
+      gst_caps_unref (gst_queue_array_pop_tail (priv->queue));
     }
-    g_queue_push_tail (priv->queue, new_caps);
+    gst_queue_array_push_tail (priv->queue, new_caps);
     gst_caps_replace (&priv->last_caps, new_caps);
   }
 
@@ -1610,7 +1650,7 @@ gst_app_src_set_latencies (GstAppSrc * appsrc, gboolean do_min, guint64 min,
  * gst_app_src_set_latency:
  * @appsrc: a #GstAppSrc
  * @min: the min latency
- * @max: the min latency
+ * @max: the max latency
  *
  * Configure the @min and @max latency in @src. If @min is set to -1, the
  * default latency calculations for pseudo-live sources will be used.
@@ -1624,8 +1664,8 @@ gst_app_src_set_latency (GstAppSrc * appsrc, guint64 min, guint64 max)
 /**
  * gst_app_src_get_latency:
  * @appsrc: a #GstAppSrc
- * @min: the min latency
- * @max: the min latency
+ * @min: (out): the min latency
+ * @max: (out): the max latency
  *
  * Retrieve the min and max latencies in @min and @max respectively.
  */
@@ -1696,17 +1736,28 @@ gst_app_src_get_emit_signals (GstAppSrc * appsrc)
 }
 
 static GstFlowReturn
-gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
-    gboolean steal_ref)
+gst_app_src_push_internal (GstAppSrc * appsrc, GstBuffer * buffer,
+    GstBufferList * buflist, gboolean steal_ref)
 {
   gboolean first = TRUE;
   GstAppSrcPrivate *priv;
 
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), GST_FLOW_ERROR);
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), GST_FLOW_ERROR);
 
   priv = appsrc->priv;
 
+  if (buffer != NULL)
+    g_return_val_if_fail (GST_IS_BUFFER (buffer), GST_FLOW_ERROR);
+  else
+    g_return_val_if_fail (GST_IS_BUFFER_LIST (buflist), GST_FLOW_ERROR);
+
+  if (buflist != NULL) {
+    if (gst_buffer_list_length (buflist) == 0)
+      return GST_FLOW_OK;
+
+    buffer = gst_buffer_list_get (buflist, 0);
+  }
+
   if (GST_BUFFER_DTS (buffer) == GST_CLOCK_TIME_NONE &&
       GST_BUFFER_PTS (buffer) == GST_CLOCK_TIME_NONE &&
       gst_base_src_get_do_timestamp (GST_BASE_SRC_CAST (appsrc))) {
@@ -1725,14 +1776,25 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
         now = 0;
       gst_object_unref (clock);
 
-      if (!steal_ref)
-        buffer = gst_buffer_copy (buffer);
-      else
-        buffer = gst_buffer_make_writable (buffer);
+      if (buflist == NULL) {
+        if (!steal_ref) {
+          buffer = gst_buffer_copy (buffer);
+          steal_ref = TRUE;
+        } else {
+          buffer = gst_buffer_make_writable (buffer);
+        }
+      } else {
+        if (!steal_ref) {
+          buflist = gst_buffer_list_copy (buflist);
+          steal_ref = TRUE;
+        } else {
+          buflist = gst_buffer_list_make_writable (buflist);
+        }
+        buffer = gst_buffer_list_get_writable (buflist, 0);
+      }
 
       GST_BUFFER_PTS (buffer) = now;
       GST_BUFFER_DTS (buffer) = now;
-      steal_ref = TRUE;
     } else {
       GST_WARNING_OBJECT (appsrc,
           "do-timestamp=TRUE but buffers are provided before "
@@ -1778,7 +1840,9 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
         GST_DEBUG_OBJECT (appsrc, "waiting for free space");
         /* we are filled, wait until a buffer gets popped or when we
          * flush. */
+        priv->wait_status = APP_WAITING;
         g_cond_wait (&priv->cond, &priv->mutex);
+        priv->wait_status = NOONE_WAITING;
       } else {
         /* no need to wait for free space, we just pump more data into the
          * queue hoping that the caller reacts to the enough-data signal and
@@ -1789,12 +1853,23 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
       break;
   }
 
-  GST_DEBUG_OBJECT (appsrc, "queueing buffer %p", buffer);
-  if (!steal_ref)
-    gst_buffer_ref (buffer);
-  g_queue_push_tail (priv->queue, buffer);
-  priv->queued_bytes += gst_buffer_get_size (buffer);
-  g_cond_broadcast (&priv->cond);
+  if (buflist != NULL) {
+    GST_DEBUG_OBJECT (appsrc, "queueing buffer list %p", buflist);
+    if (!steal_ref)
+      gst_buffer_list_ref (buflist);
+    gst_queue_array_push_tail (priv->queue, buflist);
+    priv->queued_bytes += gst_buffer_list_calculate_size (buflist);
+  } else {
+    GST_DEBUG_OBJECT (appsrc, "queueing buffer %p", buffer);
+    if (!steal_ref)
+      gst_buffer_ref (buffer);
+    gst_queue_array_push_tail (priv->queue, buffer);
+    priv->queued_bytes += gst_buffer_get_size (buffer);
+  }
+
+  if (priv->wait_status == STREAM_WAITING)
+    g_cond_broadcast (&priv->cond);
+
   g_mutex_unlock (&priv->mutex);
 
   return GST_FLOW_OK;
@@ -1818,9 +1893,17 @@ eos:
   }
 }
 
+static GstFlowReturn
+gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
+    gboolean steal_ref)
+{
+  return gst_app_src_push_internal (appsrc, buffer, NULL, steal_ref);
+}
+
 static GstFlowReturn
 gst_app_src_push_sample_internal (GstAppSrc * appsrc, GstSample * sample)
 {
+  GstBufferList *buffer_list;
   GstBuffer *buffer;
   GstCaps *caps;
 
@@ -1834,12 +1917,15 @@ gst_app_src_push_sample_internal (GstAppSrc * appsrc, GstSample * sample)
   }
 
   buffer = gst_sample_get_buffer (sample);
-  if (buffer == NULL) {
-    GST_WARNING_OBJECT (appsrc, "received sample without buffer");
-    return GST_FLOW_OK;
-  }
+  if (buffer != NULL)
+    return gst_app_src_push_buffer_full (appsrc, buffer, FALSE);
 
-  return gst_app_src_push_buffer_full (appsrc, buffer, FALSE);
+  buffer_list = gst_sample_get_buffer_list (sample);
+  if (buffer_list != NULL)
+    return gst_app_src_push_internal (appsrc, NULL, buffer_list, FALSE);
+
+  GST_WARNING_OBJECT (appsrc, "received sample without buffer or buffer list");
+  return GST_FLOW_OK;
 }
 
 /**
@@ -1863,6 +1949,30 @@ gst_app_src_push_buffer (GstAppSrc * appsrc, GstBuffer * buffer)
   return gst_app_src_push_buffer_full (appsrc, buffer, TRUE);
 }
 
+/**
+ * gst_app_src_push_buffer_list:
+ * @appsrc: a #GstAppSrc
+ * @buffer_list: (transfer full): a #GstBufferList to push
+ *
+ * Adds a buffer list to the queue of buffers and buffer lists that the
+ * appsrc element will push to its source pad.  This function takes ownership
+ * of @buffer_list.
+ *
+ * When the block property is TRUE, this function can block until free
+ * space becomes available in the queue.
+ *
+ * Returns: #GST_FLOW_OK when the buffer list was successfuly queued.
+ * #GST_FLOW_FLUSHING when @appsrc is not PAUSED or PLAYING.
+ * #GST_FLOW_EOS when EOS occured.
+ *
+ * Since: 1.14
+ */
+GstFlowReturn
+gst_app_src_push_buffer_list (GstAppSrc * appsrc, GstBufferList * buffer_list)
+{
+  return gst_app_src_push_internal (appsrc, NULL, buffer_list, TRUE);
+}
+
 /**
  * gst_app_src_push_sample:
  * @appsrc: a #GstAppSrc
@@ -1898,6 +2008,15 @@ gst_app_src_push_buffer_action (GstAppSrc * appsrc, GstBuffer * buffer)
   return gst_app_src_push_buffer_full (appsrc, buffer, FALSE);
 }
 
+/* push a buffer list without stealing the ref of the buffer list. This is
+ * used for the action signal. */
+static GstFlowReturn
+gst_app_src_push_buffer_list_action (GstAppSrc * appsrc,
+    GstBufferList * buffer_list)
+{
+  return gst_app_src_push_internal (appsrc, NULL, buffer_list, FALSE);
+}
+
 /* push a sample without stealing the ref. This is used for the
  * action signal. */
 static GstFlowReturn
