@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* TODO:
@@ -36,49 +36,53 @@
 
 /**
  * SECTION:gstaudiocdsrc
+ * @title: GstAudioCdSrc
  * @short_description: Base class for Audio CD sources
  *
- * <refsect2>
- * <para>
  * Provides a base class for CD digital audio (CDDA) sources, which handles
  * things like seeking, querying, discid calculation, tags, and buffer
  * timestamping.
- * </para>
- * <title>Using GstAudioCdSrc-based elements in applications</title>
- * <para>
+ *
+ * ## Using GstAudioCdSrc-based elements in applications
+ *
  * GstAudioCdSrc registers two #GstFormat<!-- -->s of its own, namely
  * the "track" format and the "sector" format. Applications will usually
  * only find the "track" format interesting. You can retrieve that #GstFormat
  * for use in seek events or queries with gst_format_get_by_nick("track").
- * </para>
- * <para>
+ *
  * In order to query the number of tracks, for example, an application would
  * set the CDDA source element to READY or PAUSED state and then query the
  * the number of tracks via gst_element_query_duration() using the track
  * format acquired above. Applications can query the currently playing track
  * in the same way.
- * </para>
- * <para>
+ *
  * Alternatively, applications may retrieve the currently playing track and
  * the total number of tracks from the taglist that will posted on the bus
  * whenever the CD is opened or the currently playing track changes. The
  * taglist will contain GST_TAG_TRACK_NUMBER and GST_TAG_TRACK_COUNT tags.
- * </para>
- * <para>
+ *
  * Applications playing back CD audio using playbin and cdda://n URIs should
  * issue a seek command in track format to change between tracks, rather than
  * setting a new cdda://n+1 URI on playbin (as setting a new URI on playbin
  * involves closing and re-opening the CD device, which is much much slower).
- * </para>
- * <title>Tags and meta-information</title>
- * <para>
+ *
+ * ## Tags and meta-information
+ *
  * CDDA sources will automatically emit a number of tags, details about which
  * can be found in the libgsttag documentation. Those tags are:
  * #GST_TAG_CDDA_CDDB_DISCID, #GST_TAG_CDDA_CDDB_DISCID_FULL,
  * #GST_TAG_CDDA_MUSICBRAINZ_DISCID, #GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL,
  * among others.
- * </para>
- * </refsect2>
+ *
+ * ## Tracks and Table of Contents (TOC)
+ *
+ * Applications will be informed of the available tracks via a TOC message
+ * on the pipeline's #GstBus. The #GstToc will contain a #GstTocEntry for
+ * each track, with information about each track. The duration for each
+ * track can be retrieved via the #GST_TAG_DURATION tag from each entry's
+ * tag list, or calculated via gst_toc_entry_get_start_stop_times().
+ * The track entries in the TOC will be sorted by track number.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -196,24 +200,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
 static GstFormat track_format;
 static GstFormat sector_format;
 
-GType
-gst_audio_cd_src_mode_get_type (void)
-{
-  static GType mode_type;       /* 0 */
-  static const GEnumValue modes[] = {
-    {GST_AUDIO_CD_SRC_MODE_NORMAL, "Stream consists of a single track",
-        "normal"},
-    {GST_AUDIO_CD_SRC_MODE_CONTINUOUS, "Stream consists of the whole disc",
-        "continuous"},
-    {0, NULL, NULL}
-  };
-
-  if (mode_type == 0)
-    mode_type = g_enum_register_static ("GstAudioCdSrcMode", modes);
-
-  return mode_type;
-}
-
 static void
 gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
 {
@@ -278,8 +264,8 @@ gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 #endif
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_cd_src_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_audio_cd_src_src_template);
 
 #if 0
   element_class->set_index = GST_DEBUG_FUNCPTR (gst_audio_cd_src_set_index);
@@ -506,7 +492,7 @@ gst_audio_cd_src_convert (GstAudioCdSrc * src, GstFormat src_format,
       goto wrong_value;
     }
     src_format = GST_FORMAT_DEFAULT;
-    src_val = src->priv->tracks[src_val].start * SAMPLES_PER_SECTOR;
+    src_val = src->priv->tracks[src_val].start * (gint64) SAMPLES_PER_SECTOR;
   } else if (src_format == sector_format) {
     src_format = GST_FORMAT_DEFAULT;
     src_val = src_val * SAMPLES_PER_SECTOR;
@@ -952,6 +938,7 @@ gst_audio_cd_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
         ret = gst_audio_cd_src_handle_track_seek (src, 1.0, GST_SEEK_FLAG_FLUSH,
             GST_SEEK_TYPE_SET, track_num, GST_SEEK_TYPE_NONE, -1);
       }
+      g_free (uid);
       break;
     }
     default:{
@@ -1089,7 +1076,7 @@ gst_audio_cd_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
  * gst_audio_cd_src_add_track:
  * @src: a #GstAudioCdSrc
  * @track: address of #GstAudioCdSrcTrack to add
- * 
+ *
  * CDDA sources use this function from their start vfunc to announce the
  * available data and audio tracks to the base source class. The caller
  * should allocate @track on the stack, the base source will do a shallow
@@ -1176,13 +1163,23 @@ gst_audio_cd_src_calculate_musicbrainz_discid (GstAudioCdSrc * src)
   gchar *ptr;
   gchar tmp[9];
   gulong i;
+  unsigned int last_audio_track;
   guint leadout_sector;
   gsize digest_len;
 
   s = g_string_new (NULL);
 
+  /* MusicBrainz doesn't consider trailing data tracks
+   * data tracks up front stay, since the disc has to start with 1 */
+  last_audio_track = 0;
+  for (i = 0; i < src->priv->num_tracks; i++) {
+    if (src->priv->tracks[i].is_audio) {
+      last_audio_track = src->priv->tracks[i].num;
+    }
+  }
+
   leadout_sector =
-      src->priv->tracks[src->priv->num_tracks - 1].end + 1 + CD_MSF_OFFSET;
+      src->priv->tracks[last_audio_track - 1].end + 1 + CD_MSF_OFFSET;
 
   /* generate SHA digest */
   sha = g_checksum_new (G_CHECKSUM_SHA1);
@@ -1190,10 +1187,8 @@ gst_audio_cd_src_calculate_musicbrainz_discid (GstAudioCdSrc * src)
   g_string_append_printf (s, "%02X", src->priv->tracks[0].num);
   g_checksum_update (sha, (guchar *) tmp, 2);
 
-  g_snprintf (tmp, sizeof (tmp), "%02X",
-      src->priv->tracks[src->priv->num_tracks - 1].num);
-  g_string_append_printf (s, " %02X",
-      src->priv->tracks[src->priv->num_tracks - 1].num);
+  g_snprintf (tmp, sizeof (tmp), "%02X", last_audio_track);
+  g_string_append_printf (s, " %02X", last_audio_track);
   g_checksum_update (sha, (guchar *) tmp, 2);
 
   g_snprintf (tmp, sizeof (tmp), "%08X", leadout_sector);
@@ -1201,7 +1196,7 @@ gst_audio_cd_src_calculate_musicbrainz_discid (GstAudioCdSrc * src)
   g_checksum_update (sha, (guchar *) tmp, 8);
 
   for (i = 0; i < 99; i++) {
-    if (i < src->priv->num_tracks) {
+    if (i < last_audio_track) {
       guint frame_offset = src->priv->tracks[i].start + CD_MSF_OFFSET;
 
       g_snprintf (tmp, sizeof (tmp), "%08X", frame_offset);
