@@ -19,21 +19,18 @@
 
 /**
  * SECTION:gstpbutilsdescriptions
+ * @title: Descriptions
  * @short_description: Provides human-readable descriptions for caps/codecs
  * and encoder, decoder, URI source and URI sink elements
  *
- * <refsect2>
- * <para>
  * The above functions provide human-readable strings for media formats
  * and decoder/demuxer/depayloader/encoder/muxer/payloader elements for use
  * in error dialogs or other messages shown to users.
- * </para>
- * <para>
+ *
  * gst_pb_utils_add_codec_description_to_tag_list() is a utility function
  * for demuxer and decoder elements to add audio/video codec tags from a
  * given (fixed) #GstCaps.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -287,6 +284,7 @@ static const FormatInfo formats[] = {
   {"application/x-teletext", "Teletext", 0, ""},
   {"application/x-kate", "Kate", 0, ""},
   {"subtitle/x-kate", N_("Kate subtitle format"), FLAG_SUB, ""},
+  {"application/x-subtitle-vtt", N_("WebVTT subtitle format"), FLAG_SUB, ""},
   {"subpicture/x-dvb", "DVB subtitles", FLAG_SUB, ""},
   {"subpicture/x-pgs", "PGS subtitles", FLAG_SUB, ""},
   {"subpicture/x-xsub", "XSUB subtitles", FLAG_SUB, ""},
@@ -436,6 +434,8 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
 
   g_assert (info != NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   if (info->desc != NULL)
     return g_strdup (_(info->desc));
 
@@ -932,6 +932,8 @@ gst_pb_utils_get_source_description (const gchar * protocol)
 
   g_return_val_if_fail (protocol != NULL, NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   if (strcmp (protocol, "cdda") == 0)
     return g_strdup (_("Audio CD source"));
 
@@ -1021,6 +1023,8 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
 
   g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP depayloader"), str);
@@ -1071,6 +1075,7 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
   g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  gst_pb_utils_init_locale_text_domain ();
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -1119,6 +1124,8 @@ gst_pb_utils_get_element_description (const gchar * factory_name)
 
   g_return_val_if_fail (factory_name != NULL, NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   ret = g_strdup_printf (_("GStreamer element %s"), factory_name);
   if (ret && g_str_has_prefix (ret, factory_name))
     *ret = g_ascii_toupper (*ret);
