@@ -71,6 +71,17 @@ G_BEGIN_DECLS
  */
 #define GST_ENCODING_CATEGORY_CAPTURE           "capture"
 
+/**
+ * GST_ENCODING_CATEGORY_FILE_EXTENSION:
+ *
+ * #GstEncodingTarget category for file extensions.
+ * The name of the target will be the name of the file extensions possible
+ * for a particular target. Those targets are defining like 'default' formats
+ * usually used for a particular file extension.
+ */
+
+#define GST_ENCODING_CATEGORY_FILE_EXTENSION    "file-extension"
+
 /**
  * GstEncodingTarget:
  *
@@ -89,6 +100,7 @@ G_BEGIN_DECLS
 typedef struct _GstEncodingTarget GstEncodingTarget;
 typedef GObjectClass GstEncodingTargetClass;
 
+GST_PBUTILS_API
 GType gst_encoding_target_get_type (void);
 
 /**
@@ -109,35 +121,60 @@ GType gst_encoding_target_get_type (void);
 #define gst_encoding_target_ref(target) \
   (g_object_ref ((GObject*) target))
 
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_new                 (const gchar *name,
                                                                  const gchar *category,
                                                                  const gchar *description,
                                                                  const GList *profiles);
 
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_name            (GstEncodingTarget *target);
+
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_category        (GstEncodingTarget *target);
+
+GST_PBUTILS_API
 const gchar *           gst_encoding_target_get_description     (GstEncodingTarget *target);
+
+GST_PBUTILS_API
 const GList *           gst_encoding_target_get_profiles        (GstEncodingTarget *target);
+
+GST_PBUTILS_API
 GstEncodingProfile *    gst_encoding_target_get_profile         (GstEncodingTarget *target,
                                                                  const gchar *name);
 
+GST_PBUTILS_API
 gboolean                gst_encoding_target_add_profile         (GstEncodingTarget *target,
                                                                  GstEncodingProfile *profile);
 
+GST_PBUTILS_API
 gboolean                gst_encoding_target_save                (GstEncodingTarget *target,
                                                                  GError **error);
+
+GST_PBUTILS_API
 gboolean                gst_encoding_target_save_to_file        (GstEncodingTarget *target,
                                                                  const gchar *filepath,
                                                                  GError **error);
+
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_load                (const gchar *name,
                                                                  const gchar *category,
                                                                  GError **error);
+
+GST_PBUTILS_API
 GstEncodingTarget *     gst_encoding_target_load_from_file      (const gchar *filepath,
                                                                  GError **error);
 
+GST_PBUTILS_API
 GList *                 gst_encoding_list_available_categories  (void);
+
+GST_PBUTILS_API
 GList *                 gst_encoding_list_all_targets           (const gchar * categoryname);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEncodingTarget, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif  /* __GST_PROFILE_REGISTRY_H__ */
