@@ -20,6 +20,7 @@
 
 /**
  * SECTION:gstdiscoverer
+ * @title: GstDiscoverer
  * @short_description: Utility for discovering information on URIs.
  *
  * The #GstDiscoverer is a utility object which allows to get as much
@@ -57,6 +58,7 @@ GST_DEBUG_CATEGORY_STATIC (discoverer_debug);
 
 static GQuark _CAPS_QUARK;
 static GQuark _TAGS_QUARK;
+static GQuark _ELEMENT_SRCPAD_QUARK;
 static GQuark _TOC_QUARK;
 static GQuark _STREAM_ID_QUARK;
 static GQuark _MISSING_PLUGIN_QUARK;
@@ -73,6 +75,7 @@ typedef struct
   GstTagList *tags;
   GstToc *toc;
   gchar *stream_id;
+  gulong probe_id;
 } PrivateStream;
 
 struct _GstDiscovererPrivate
@@ -86,6 +89,8 @@ struct _GstDiscovererPrivate
   GList *pending_uris;
 
   GMutex lock;
+  /* TRUE if cleaning up discoverer */
+  gboolean cleanup;
 
   /* TRUE if processing a URI */
   gboolean processing;
@@ -93,9 +98,6 @@ struct _GstDiscovererPrivate
   /* TRUE if discoverer has been started */
   gboolean running;
 
-  /* TRUE if ASYNC_DONE has been received (need to check for subtitle tags) */
-  gboolean async_done;
-
   /* current items */
   GstDiscovererInfo *current_info;
   GError *current_error;
@@ -107,6 +109,12 @@ struct _GstDiscovererPrivate
   /* List of these sinks and their handler IDs (to remove the probe) */
   guint pending_subtitle_pads;
 
+  /* Whether we received no_more_pads */
+  gboolean no_more_pads;
+
+  GstState target_state;
+  GstState current_state;
+
   /* Global elements */
   GstBin *pipeline;
   GstElement *uridecodebin;
@@ -125,6 +133,7 @@ struct _GstDiscovererPrivate
   /* Handler ids for various callbacks */
   gulong pad_added_id;
   gulong pad_remove_id;
+  gulong no_more_pads_id;
   gulong source_chg_id;
   gulong element_added_id;
   gulong bus_cb_id;
@@ -139,6 +148,7 @@ _do_init (void)
   GST_DEBUG_CATEGORY_INIT (discoverer_debug, "discoverer", 0, "Discoverer");
 
   _CAPS_QUARK = g_quark_from_static_string ("caps");
+  _ELEMENT_SRCPAD_QUARK = g_quark_from_static_string ("element-srcpad");
   _TAGS_QUARK = g_quark_from_static_string ("tags");
   _TOC_QUARK = g_quark_from_static_string ("toc");
   _STREAM_ID_QUARK = g_quark_from_static_string ("stream-id");
@@ -179,6 +189,8 @@ static void uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
     GstDiscoverer * dc);
 static void uridecodebin_pad_removed_cb (GstElement * uridecodebin,
     GstPad * pad, GstDiscoverer * dc);
+static void uridecodebin_no_more_pads_cb (GstElement * uridecodebin,
+    GstDiscoverer * dc);
 static void uridecodebin_source_changed_cb (GstElement * uridecodebin,
     GParamSpec * pspec, GstDiscoverer * dc);
 
@@ -244,10 +256,11 @@ gst_discoverer_class_init (GstDiscovererClass * klass)
    * GstDiscoverer::discovered:
    * @discoverer: the #GstDiscoverer
    * @info: the results #GstDiscovererInfo
-   * @error: (type GLib.Error): #GError, which will be non-NULL if an error
-   *                            occurred during discovery. You must not
-   *                            free this #GError, it will be freed by
-   *                            the discoverer.
+   * @error: (allow-none) (type GLib.Error): #GError, which will be non-NULL
+   *                                         if an error occurred during
+   *                                         discovery. You must not free
+   *                                         this #GError, it will be freed by
+   *                                         the discoverer.
    *
    * Will be emitted in async mode when all information on a URI could be
    * discovered, or an error occurred.
@@ -304,12 +317,15 @@ gst_discoverer_init (GstDiscoverer * dc)
 
   dc->priv->timeout = DEFAULT_PROP_TIMEOUT;
   dc->priv->async = FALSE;
-  dc->priv->async_done = FALSE;
 
   g_mutex_init (&dc->priv->lock);
 
   dc->priv->pending_subtitle_pads = 0;
 
+  dc->priv->current_state = GST_STATE_NULL;
+  dc->priv->target_state = GST_STATE_NULL;
+  dc->priv->no_more_pads = FALSE;
+
   GST_LOG ("Creating pipeline");
   dc->priv->pipeline = (GstBin *) gst_pipeline_new ("Discoverer");
   GST_LOG_OBJECT (dc, "Creating uridecodebin");
@@ -328,6 +344,9 @@ gst_discoverer_init (GstDiscoverer * dc)
   dc->priv->pad_remove_id =
       g_signal_connect_object (dc->priv->uridecodebin, "pad-removed",
       G_CALLBACK (uridecodebin_pad_removed_cb), dc, 0);
+  dc->priv->no_more_pads_id =
+      g_signal_connect_object (dc->priv->uridecodebin, "no-more-pads",
+      G_CALLBACK (uridecodebin_no_more_pads_cb), dc, 0);
   dc->priv->source_chg_id =
       g_signal_connect_object (dc->priv->uridecodebin, "notify::source",
       G_CALLBACK (uridecodebin_source_changed_cb), dc, 0);
@@ -389,13 +408,15 @@ gst_discoverer_dispose (GObject * obj)
     /* Workaround for bug #118536 */
     DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->pad_added_id);
     DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->pad_remove_id);
+    DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->no_more_pads_id);
     DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->source_chg_id);
     DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->element_added_id);
     DISCONNECT_SIGNAL (dc->priv->bus, dc->priv->bus_cb_id);
 
     /* pipeline was set to NULL in _reset */
     gst_object_unref (dc->priv->pipeline);
-    gst_object_unref (dc->priv->bus);
+    if (dc->priv->bus)
+      gst_object_unref (dc->priv->bus);
 
     dc->priv->pipeline = NULL;
     dc->priv->uridecodebin = NULL;
@@ -547,6 +568,7 @@ is_subtitle_caps (const GstCaps * caps)
 static GstPadProbeReturn
 got_subtitle_data (GstPad * pad, GstPadProbeInfo * info, GstDiscoverer * dc)
 {
+  GstMessage *msg;
 
   if (!(GST_IS_BUFFER (info->data) || (GST_IS_EVENT (info->data)
               && (GST_EVENT_TYPE ((GstEvent *) info->data) == GST_EVENT_GAP
@@ -559,11 +581,10 @@ got_subtitle_data (GstPad * pad, GstPadProbeInfo * info, GstDiscoverer * dc)
 
   dc->priv->pending_subtitle_pads--;
 
-  if (dc->priv->pending_subtitle_pads == 0) {
-    GstMessage *msg = gst_message_new_application (NULL,
-        gst_structure_new_empty ("DiscovererDone"));
-    gst_element_post_message ((GstElement *) dc->priv->pipeline, msg);
-  }
+  msg = gst_message_new_application (NULL,
+      gst_structure_new_empty ("DiscovererDone"));
+  gst_element_post_message ((GstElement *) dc->priv->pipeline, msg);
+
   DISCO_UNLOCK (dc);
 
   return GST_PAD_PROBE_REMOVE;
@@ -591,15 +612,34 @@ uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
   PrivateStream *ps;
   GstPad *sinkpad = NULL;
   GstCaps *caps;
+  gchar *padname;
+  gchar *tmpname;
 
   GST_DEBUG_OBJECT (dc, "pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
+  DISCO_LOCK (dc);
+  if (dc->priv->cleanup) {
+    GST_WARNING_OBJECT (dc, "Cleanup, not adding pad");
+    DISCO_UNLOCK (dc);
+    return;
+  }
+  if (dc->priv->current_error) {
+    GST_WARNING_OBJECT (dc, "Ongoing error, not adding more pads");
+    DISCO_UNLOCK (dc);
+    return;
+  }
   ps = g_slice_new0 (PrivateStream);
 
   ps->dc = dc;
   ps->pad = pad;
-  ps->queue = gst_element_factory_make ("queue", NULL);
-  ps->sink = gst_element_factory_make ("fakesink", NULL);
+  padname = gst_pad_get_name (pad);
+  tmpname = g_strdup_printf ("discoverer-queue-%s", padname);
+  ps->queue = gst_element_factory_make ("queue", tmpname);
+  g_free (tmpname);
+  tmpname = g_strdup_printf ("discoverer-sink-%s", padname);
+  ps->sink = gst_element_factory_make ("fakesink", tmpname);
+  g_free (tmpname);
+  g_free (padname);
 
   if (G_UNLIKELY (ps->queue == NULL || ps->sink == NULL))
     goto error;
@@ -616,12 +656,11 @@ uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
   if (is_subtitle_caps (caps)) {
     /* Subtitle streams are sparse and may not provide any information - don't
      * wait for data to preroll */
-    gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_DATA_DOWNSTREAM,
+    ps->probe_id =
+        gst_pad_add_probe (sinkpad, GST_PAD_PROBE_TYPE_DATA_DOWNSTREAM,
         (GstPadProbeCallback) got_subtitle_data, dc, NULL);
     g_object_set (ps->sink, "async", FALSE, NULL);
-    DISCO_LOCK (dc);
     dc->priv->pending_subtitle_pads++;
-    DISCO_UNLOCK (dc);
   }
 
   gst_caps_unref (caps);
@@ -645,7 +684,6 @@ uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
   gst_pad_add_probe (pad, GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM,
       (GstPadProbeCallback) _event_probe, ps, NULL);
 
-  DISCO_LOCK (dc);
   dc->priv->streams = g_list_append (dc->priv->streams, ps);
   DISCO_UNLOCK (dc);
 
@@ -662,9 +700,22 @@ error:
   if (ps->sink)
     gst_object_unref (ps->sink);
   g_slice_free (PrivateStream, ps);
+  DISCO_UNLOCK (dc);
   return;
 }
 
+static void
+uridecodebin_no_more_pads_cb (GstElement * uridecodebin, GstDiscoverer * dc)
+{
+  GstMessage *msg = gst_message_new_application (NULL,
+      gst_structure_new_empty ("DiscovererDone"));
+
+  DISCO_LOCK (dc);
+  dc->priv->no_more_pads = TRUE;
+  gst_element_post_message ((GstElement *) dc->priv->pipeline, msg);
+  DISCO_UNLOCK (dc);
+}
+
 static void
 uridecodebin_pad_removed_cb (GstElement * uridecodebin, GstPad * pad,
     GstDiscoverer * dc)
@@ -689,8 +740,10 @@ uridecodebin_pad_removed_cb (GstElement * uridecodebin, GstPad * pad,
     return;
   }
 
+  if (ps->probe_id)
+    gst_pad_remove_probe (pad, ps->probe_id);
+
   dc->priv->streams = g_list_delete_link (dc->priv->streams, tmp);
-  DISCO_UNLOCK (dc);
 
   gst_element_set_state (ps->sink, GST_STATE_NULL);
   gst_element_set_state (ps->queue, GST_STATE_NULL);
@@ -703,6 +756,7 @@ uridecodebin_pad_removed_cb (GstElement * uridecodebin, GstPad * pad,
   /* references removed here */
   gst_bin_remove_many (dc->priv->pipeline, ps->sink, ps->queue, NULL);
 
+  DISCO_UNLOCK (dc);
   if (ps->tags) {
     gst_tag_list_unref (ps->tags);
   }
@@ -803,19 +857,28 @@ static GstDiscovererStreamInfo *
 collect_information (GstDiscoverer * dc, const GstStructure * st,
     GstDiscovererStreamInfo * parent)
 {
-  GstCaps *caps;
+  GstPad *srcpad;
+  GstCaps *caps = NULL;
   GstStructure *caps_st;
   GstTagList *tags_st;
   const gchar *name;
   gint tmp, tmp2;
   guint utmp;
 
-  if (!st || !gst_structure_id_has_field (st, _CAPS_QUARK)) {
+  if (!st || (!gst_structure_id_has_field (st, _CAPS_QUARK)
+          && !gst_structure_id_has_field (st, _ELEMENT_SRCPAD_QUARK))) {
     GST_WARNING ("Couldn't find caps !");
     return make_info (parent, GST_TYPE_DISCOVERER_STREAM_INFO, NULL);
   }
 
-  gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+  if (gst_structure_id_get (st, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD, &srcpad,
+          NULL)) {
+    caps = gst_pad_get_current_caps (srcpad);
+    gst_object_unref (srcpad);
+  }
+  if (!caps) {
+    gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+  }
 
   if (!caps || gst_caps_is_empty (caps) || gst_caps_is_any (caps)) {
     GST_WARNING ("Couldn't find caps !");
@@ -830,6 +893,7 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
   if (g_str_has_prefix (name, "audio/")) {
     GstDiscovererAudioInfo *info;
     const gchar *format_str;
+    guint64 channel_mask;
 
     info = (GstDiscovererAudioInfo *) make_info (parent,
         GST_TYPE_DISCOVERER_AUDIO_INFO, caps);
@@ -840,6 +904,13 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
     if (gst_structure_get_int (caps_st, "channels", &tmp))
       info->channels = (guint) tmp;
 
+    if (gst_structure_get (caps_st, "channel-mask", GST_TYPE_BITMASK,
+            &channel_mask, NULL)) {
+      info->channel_mask = channel_mask;
+    } else if (info->channels) {
+      info->channel_mask = gst_audio_channel_get_fallback_mask (info->channels);
+    }
+
     /* FIXME: we only want to extract depth if raw audio is what's in the
      * container (i.e. not if there is a decoder involved) */
     format_str = gst_structure_get_string (caps_st, "format");
@@ -1022,6 +1093,8 @@ find_stream_for_node (GstDiscoverer * dc, const GstStructure * topology)
     ps = (PrivateStream *) tmp->data;
 
     target_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (ps->pad));
+    if (target_pad == NULL)
+      continue;
     gst_object_unref (target_pad);
 
     if (target_pad == pad)
@@ -1042,7 +1115,6 @@ static gboolean
 child_is_same_stream (const GstCaps * _parent, const GstCaps * child)
 {
   GstCaps *parent;
-  guint i, size;
   gboolean res;
 
   if (_parent == child)
@@ -1052,13 +1124,7 @@ child_is_same_stream (const GstCaps * _parent, const GstCaps * child)
   if (!child)
     return FALSE;
 
-  parent = gst_caps_copy (_parent);
-  size = gst_caps_get_size (parent);
-
-  for (i = 0; i < size; i++) {
-    gst_structure_remove_field (gst_caps_get_structure (parent, i), "parsed");
-    gst_structure_remove_field (gst_caps_get_structure (parent, i), "framed");
-  }
+  parent = copy_and_clean_caps (_parent);
   res = gst_caps_can_intersect (parent, child);
   gst_caps_unref (parent);
   return res;
@@ -1129,6 +1195,8 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
       /* FIXME : aggregate with information from main streams */
       GST_DEBUG ("Coudn't find 'next' ! might be the last entry");
     } else {
+      GstPad *srcpad;
+
       st = (GstStructure *) gst_value_get_structure (nval);
 
       GST_DEBUG ("next is a structure %" GST_PTR_FORMAT, st);
@@ -1136,7 +1204,16 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
       if (!parent)
         parent = res;
 
-      if (gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL)) {
+      if (gst_structure_id_get (st, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD,
+              &srcpad, NULL)) {
+        caps = gst_pad_get_current_caps (srcpad);
+        gst_object_unref (srcpad);
+      }
+      if (!caps) {
+        gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+      }
+
+      if (caps) {
         if (child_is_same_stream (parent->caps, caps)) {
           /* We sometimes get an extra sub-stream from the parser. If this is
            * the case, we just replace the parent caps with this stream's caps
@@ -1155,8 +1232,7 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
           res->next = next;
           next->previous = res;
         }
-        if (caps)
-          gst_caps_unref (caps);
+        gst_caps_unref (caps);
       }
     }
 
@@ -1171,9 +1247,18 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
     guint i, len;
     GstDiscovererContainerInfo *cont;
     GstTagList *tags;
+    GstPad *srcpad;
 
-    if (!gst_structure_id_get (topology, _CAPS_QUARK,
-            GST_TYPE_CAPS, &caps, NULL))
+    if (gst_structure_id_get (topology, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD,
+            &srcpad, NULL)) {
+      caps = gst_pad_get_current_caps (srcpad);
+      gst_object_unref (srcpad);
+    }
+    if (!caps) {
+      gst_structure_id_get (topology, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+    }
+
+    if (!caps)
       GST_WARNING ("Couldn't find caps !");
 
     len = gst_value_list_get_size (nval);
@@ -1244,8 +1329,10 @@ discoverer_collect (GstDiscoverer * dc)
       if (gst_element_query_duration (pipeline, GST_FORMAT_TIME, &dur)) {
         GST_DEBUG ("Got duration %" GST_TIME_FORMAT, GST_TIME_ARGS (dur));
         dc->priv->current_info->duration = (guint64) dur;
-      } else {
+      } else if (dc->priv->current_info->result != GST_DISCOVERER_ERROR) {
         GstStateChangeReturn sret;
+        /* Note: We don't switch to PLAYING if we previously saw an ERROR since
+         * the state of various element isn't guaranteed anymore */
 
         /* Some parsers may not even return a rough estimate right away, e.g.
          * because they've only processed a single frame so far, so if we
@@ -1254,7 +1341,10 @@ discoverer_collect (GstDiscoverer * dc)
          * completely bogus values. We need some API extensions to solve this
          * better. */
         GST_INFO ("No duration yet, try a bit harder..");
+        /* Make sure we don't add/remove elements while switching to PLAYING itself */
+        DISCO_LOCK (dc);
         sret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
+        DISCO_UNLOCK (dc);
         if (sret != GST_STATE_CHANGE_FAILURE) {
           int i;
 
@@ -1286,6 +1376,11 @@ discoverer_collect (GstDiscoverer * dc)
       }
     }
 
+    if (dc->priv->target_state == GST_STATE_PAUSED)
+      dc->priv->current_info->live = FALSE;
+    else
+      dc->priv->current_info->live = TRUE;
+
     if (dc->priv->current_topology)
       dc->priv->current_info->stream_info = parse_stream_topology (dc,
           dc->priv->current_topology, NULL);
@@ -1362,6 +1457,7 @@ static gboolean
 handle_message (GstDiscoverer * dc, GstMessage * msg)
 {
   gboolean done = FALSE;
+  const gchar *dump_name = NULL;
 
   GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg), "got a %s message",
       GST_MESSAGE_TYPE_NAME (msg));
@@ -1379,6 +1475,7 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
 
       /* We need to stop */
       done = TRUE;
+      dump_name = "gst-discoverer-error";
 
       /* Don't override missing plugin result code for missing plugin errors */
       if (dc->priv->current_info->result != GST_DISCOVERER_MISSING_PLUGINS ||
@@ -1392,37 +1489,65 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
     }
       break;
 
-    case GST_MESSAGE_EOS:
-      GST_DEBUG ("Got EOS !");
-      done = TRUE;
-      break;
+    case GST_MESSAGE_WARNING:{
+      GError *err;
+      gchar *debug = NULL;
 
-    case GST_MESSAGE_APPLICATION:{
-      const gchar *name;
-      gboolean async_done;
-      name = gst_structure_get_name (gst_message_get_structure (msg));
-      /* Maybe ASYNC_DONE is received & we're just waiting for subtitle tags */
-      DISCO_LOCK (dc);
-      async_done = dc->priv->async_done;
-      DISCO_UNLOCK (dc);
-      if (g_str_equal (name, "DiscovererDone") && async_done)
-        return TRUE;
+      gst_message_parse_warning (msg, &err, &debug);
+      GST_WARNING_OBJECT (GST_MESSAGE_SRC (msg),
+          "Got a warning [debug:%s], [message:%s]", debug, err->message);
+      g_clear_error (&err);
+      g_free (debug);
+      dump_name = "gst-discoverer-warning";
       break;
     }
 
-    case GST_MESSAGE_ASYNC_DONE:
-      if (GST_MESSAGE_SRC (msg) == (GstObject *) dc->priv->pipeline) {
-        GST_DEBUG ("Finished changing state asynchronously");
-        DISCO_LOCK (dc);
-        if (dc->priv->pending_subtitle_pads == 0) {
-          done = TRUE;
-        } else {
-          /* Remember that ASYNC_DONE has been received, wait for subtitles */
-          dc->priv->async_done = TRUE;
-        }
-        DISCO_UNLOCK (dc);
+    case GST_MESSAGE_EOS:
+      GST_DEBUG ("Got EOS !");
+      done = TRUE;
+      dump_name = "gst-discoverer-eos";
+      break;
 
+    case GST_MESSAGE_APPLICATION:{
+      const gchar *name =
+          gst_structure_get_name (gst_message_get_structure (msg));
+
+      if (g_strcmp0 (name, "DiscovererDone"))
+        break;
+
+      /* Maybe we already reached the target state, and all we're waiting for
+       * is either the subtitle tags or no_more_pads
+       */
+      DISCO_LOCK (dc);
+      if (dc->priv->pending_subtitle_pads == 0)
+        done = dc->priv->no_more_pads
+            && dc->priv->target_state == dc->priv->current_state;
+      DISCO_UNLOCK (dc);
+
+      if (done)
+        dump_name = "gst-discoverer-application-message";
+    }
+      break;
+
+    case GST_MESSAGE_STATE_CHANGED:{
+      GstState old, new, pending;
+
+      gst_message_parse_state_changed (msg, &old, &new, &pending);
+      if (GST_MESSAGE_SRC (msg) == (GstObject *) dc->priv->pipeline) {
+        DISCO_LOCK (dc);
+        dc->priv->current_state = new;
+
+        if (dc->priv->pending_subtitle_pads == 0)
+          done = dc->priv->no_more_pads
+              && dc->priv->target_state == dc->priv->current_state;
+        /* Else we should get unblocked in GST_MESSAGE_APPLICATION */
+
+        DISCO_UNLOCK (dc);
       }
+
+      if (done)
+        dump_name = "gst-discoverer-target-state";
+    }
       break;
 
     case GST_MESSAGE_ELEMENT:
@@ -1491,6 +1616,11 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
       break;
   }
 
+  if (dump_name != NULL) {
+    /* dump graph when done or for warnings */
+    GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (dc->priv->pipeline),
+        GST_DEBUG_GRAPH_SHOW_ALL, dump_name);
+  }
   return done;
 }
 
@@ -1553,20 +1683,25 @@ _setup_locked (GstDiscoverer * dc)
 
   dc->priv->processing = TRUE;
 
+  dc->priv->target_state = GST_STATE_PAUSED;
+
   /* set pipeline to PAUSED */
   DISCO_UNLOCK (dc);
   GST_DEBUG ("Setting pipeline to PAUSED");
   ret =
       gst_element_set_state ((GstElement *) dc->priv->pipeline,
-      GST_STATE_PAUSED);
+      dc->priv->target_state);
+
   if (ret == GST_STATE_CHANGE_NO_PREROLL) {
     GST_DEBUG ("Source is live, switching to PLAYING");
+    dc->priv->target_state = GST_STATE_PLAYING;
     ret =
         gst_element_set_state ((GstElement *) dc->priv->pipeline,
-        GST_STATE_PLAYING);
+        dc->priv->target_state);
   }
   DISCO_LOCK (dc);
 
+
   GST_DEBUG_OBJECT (dc, "Pipeline going to PAUSED : %s",
       gst_element_state_change_return_get_name (ret));
 }
@@ -1576,6 +1711,10 @@ discoverer_cleanup (GstDiscoverer * dc)
 {
   GST_DEBUG ("Cleaning up");
 
+  DISCO_LOCK (dc);
+  dc->priv->cleanup = TRUE;
+  DISCO_UNLOCK (dc);
+
   gst_bus_set_flushing (dc->priv->bus, TRUE);
 
   DISCO_LOCK (dc);
@@ -1600,7 +1739,12 @@ discoverer_cleanup (GstDiscoverer * dc)
   dc->priv->current_info = NULL;
 
   dc->priv->pending_subtitle_pads = 0;
-  dc->priv->async_done = FALSE;
+
+  dc->priv->current_state = GST_STATE_NULL;
+  dc->priv->target_state = GST_STATE_NULL;
+  dc->priv->no_more_pads = FALSE;
+  dc->priv->cleanup = FALSE;
+
 
   /* Try popping the next uri */
   if (dc->priv->async) {
@@ -1732,8 +1876,8 @@ _serialize_info (GstDiscovererInfo * info, GstDiscovererSerializeFlags flags)
     tags_str = gst_tag_list_to_string (info->tags);
 
   ret =
-      g_variant_new ("(mstbms)", info->uri, info->duration, info->seekable,
-      tags_str);
+      g_variant_new ("(mstbmsb)", info->uri, info->duration, info->seekable,
+      tags_str, info->live);
 
   g_free (tags_str);
 
@@ -1743,10 +1887,11 @@ _serialize_info (GstDiscovererInfo * info, GstDiscovererSerializeFlags flags)
 static GVariant *
 _serialize_audio_stream_info (GstDiscovererAudioInfo * ainfo)
 {
-  return g_variant_new ("(uuuuums)",
+  return g_variant_new ("(uuuuumst)",
       ainfo->channels,
       ainfo->sample_rate,
-      ainfo->bitrate, ainfo->max_bitrate, ainfo->depth, ainfo->language);
+      ainfo->bitrate, ainfo->max_bitrate, ainfo->depth, ainfo->language,
+      ainfo->channel_mask);
 }
 
 static GVariant *
@@ -1859,6 +2004,8 @@ _parse_info (GstDiscovererInfo * info, GVariant * info_variant)
   str = _maybe_get_string_from_tuple (info_variant, 3);
   if (str)
     info->tags = gst_tag_list_new_from_string (str);
+
+  GET_FROM_TUPLE (info_variant, boolean, 4, &info->live);
 }
 
 static void
@@ -1901,6 +2048,8 @@ _parse_audio_stream_info (GstDiscovererAudioInfo * ainfo, GVariant * specific)
   if (str)
     ainfo->language = g_strdup (str);
 
+  GET_FROM_TUPLE (specific, uint64, 6, &ainfo->channel_mask);
+
   g_variant_unref (specific);
 }
 
