@@ -71,7 +71,7 @@ struct _GstRTPBaseDepayload
  * @packet_lost: signal the depayloader about packet loss
  * @handle_event: custom event handling
  *
- * Base class for audio RTP payloader.
+ * Base class for RTP depayloaders.
  */
 struct _GstRTPBaseDepayloadClass
 {
@@ -114,9 +114,13 @@ struct _GstRTPBaseDepayloadClass
   gpointer _gst_reserved[GST_PADDING - 1];
 };
 
+GST_RTP_API
 GType gst_rtp_base_depayload_get_type (void);
 
+GST_RTP_API
 GstFlowReturn   gst_rtp_base_depayload_push       (GstRTPBaseDepayload *filter, GstBuffer *out_buf);
+
+GST_RTP_API
 GstFlowReturn   gst_rtp_base_depayload_push_list  (GstRTPBaseDepayload *filter, GstBufferList *out_list);
 
 
