@@ -14,6 +14,7 @@
 
 /**
  * SECTION:gstrtpbasepayload
+ * @title: GstRTPBasePayload
  * @short_description: Base class for RTP payloader
  *
  * Provides a base class for RTP payloaders
@@ -275,48 +276,14 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
    * application/x-rtp-payload-stats containing the following fields relating to
    * the last processed buffer and current state of the stream being payloaded:
    *
-   * <variablelist>
-   *   <varlistentry>
-   *     <term>clock-rate</term>
-   *     <listitem><para>#G_TYPE_UINT, clock-rate of the
-   *     stream</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>running-time</term>
-   *     <listitem><para>#G_TYPE_UINT64, running time
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>seqnum</term>
-   *     <listitem><para>#G_TYPE_UINT, sequence number, same as
-   *     #GstRTPBasePayload:seqnum</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>timestamp</term>
-   *     <listitem><para>#G_TYPE_UINT, RTP timestamp, same as
-   *     #GstRTPBasePayload:timestamp</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>ssrc</term>
-   *     <listitem><para>#G_TYPE_UINT, The SSRC in use
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>pt</term>
-   *     <listitem><para>#G_TYPE_UINT, The Payload type in use, same as
-   *     #GstRTPBasePayload:pt</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>seqnum-offset</term>
-   *     <listitem><para>#G_TYPE_UINT, The current offset added to the
-   *     seqnum</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>timestamp-offset</term>
-   *     <listitem><para>#G_TYPE_UINT, The current offset added to the
-   *     timestamp</para></listitem>
-   *   </varlistentry>
-   * </variablelist>
+   *   * `clock-rate` :#G_TYPE_UINT, clock-rate of the stream
+   *   * `running-time` :#G_TYPE_UINT64, running time
+   *   * `seqnum` :#G_TYPE_UINT, sequence number, same as #GstRTPBasePayload:seqnum
+   *   * `timestamp` :#G_TYPE_UINT, RTP timestamp, same as #GstRTPBasePayload:timestamp
+   *   * `ssrc` :#G_TYPE_UINT, The SSRC in use
+   *   * `pt` :#G_TYPE_UINT, The Payload type in use, same as #GstRTPBasePayload:pt
+   *   * `seqnum-offset` :#G_TYPE_UINT, The current offset added to the seqnum
+   *   * `timestamp-offset` :#G_TYPE_UINT, The current offset added to the timestamp
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_STATS,
       g_param_spec_boxed ("stats", "Statistics", "Various statistics",
@@ -656,8 +623,16 @@ gst_rtp_base_payload_chain (GstPad * pad, GstObject * parent,
   if (!rtpbasepayload->priv->negotiated)
     goto not_negotiated;
 
-  if (gst_pad_check_reconfigure (GST_RTP_BASE_PAYLOAD_SRCPAD (rtpbasepayload)))
-    gst_rtp_base_payload_negotiate (rtpbasepayload);
+  if (gst_pad_check_reconfigure (GST_RTP_BASE_PAYLOAD_SRCPAD (rtpbasepayload))) {
+    if (!gst_rtp_base_payload_negotiate (rtpbasepayload)) {
+      gst_pad_mark_reconfigure (GST_RTP_BASE_PAYLOAD_SRCPAD (rtpbasepayload));
+      if (GST_PAD_IS_FLUSHING (GST_RTP_BASE_PAYLOAD_SRCPAD (rtpbasepayload))) {
+        goto flushing;
+      } else {
+        goto negotiate_failed;
+      }
+    }
+  }
 
   ret = rtpbasepayload_class->handle_buffer (rtpbasepayload, buffer);
 
@@ -679,6 +654,18 @@ not_negotiated:
     gst_buffer_unref (buffer);
     return GST_FLOW_NOT_NEGOTIATED;
   }
+negotiate_failed:
+  {
+    GST_DEBUG_OBJECT (rtpbasepayload, "Not negotiated");
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+flushing:
+  {
+    GST_DEBUG_OBJECT (rtpbasepayload, "we are flushing");
+    gst_buffer_unref (buffer);
+    return GST_FLOW_FLUSHING;
+  }
 }
 
 /**
@@ -827,7 +814,8 @@ gst_rtp_base_payload_negotiate (GstRTPBasePayload * payload)
     if (gst_caps_is_empty (temp)) {
       gst_caps_unref (temp);
       gst_caps_unref (templ);
-      return FALSE;
+      res = FALSE;
+      goto out;
     }
 
     /* We prefer the pt, timestamp-offset, seqnum-offset from the
@@ -1070,6 +1058,11 @@ gst_rtp_base_payload_negotiate (GstRTPBasePayload * payload)
   gst_caps_unref (srccaps);
   gst_caps_unref (templ);
 
+out:
+
+  if (!res)
+    gst_pad_mark_reconfigure (GST_RTP_BASE_PAYLOAD_SRCPAD (payload));
+
   return res;
 }
 
