@@ -20,16 +20,14 @@
 
 /**
  * SECTION:gstrtpbuffer
+ * @title: GstRTPBuffer
  * @short_description: Helper methods for dealing with RTP buffers
  * @see_also: #GstRTPBasePayload, #GstRTPBaseDepayload, gstrtcpbuffer
  *
- * <refsect2>
- * <para>
- * The GstRTPBuffer helper functions makes it easy to parse and create regular 
+ * The GstRTPBuffer helper functions makes it easy to parse and create regular
  * #GstBuffer objects that contain RTP payloads. These buffers are typically of
  * 'application/x-rtp' #GstCaps.
- * </para>
- * </refsect2>
+ *
  */
 
 #include "gstrtpbuffer.h"
@@ -181,7 +179,7 @@ gst_rtp_buffer_new_take_data (gpointer data, gsize len)
  * Returns: A newly allocated buffer with a copy of @data and of size @len.
  */
 GstBuffer *
-gst_rtp_buffer_new_copy_data (gpointer data, gsize len)
+gst_rtp_buffer_new_copy_data (gconstpointer data, gsize len)
 {
   return gst_rtp_buffer_new_take_data (g_memdup (data, len), len);
 }
@@ -323,12 +321,14 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
   guint size;
   gsize bufsize, skip;
   guint idx, length;
+  guint n_mem;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
   g_return_val_if_fail (rtp != NULL, FALSE);
   g_return_val_if_fail (rtp->buffer == NULL, FALSE);
 
-  if (gst_buffer_n_memory (buffer) < 1)
+  n_mem = gst_buffer_n_memory (buffer);
+  if (n_mem < 1)
     goto no_memory;
 
   /* map first memory, this should be the header */
@@ -397,8 +397,9 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
     rtp->size[1] = 0;
   }
 
-  /* check for padding */
-  if (data[0] & 0x20) {
+  /* check for padding unless flags says to skip */
+  if ((data[0] & 0x20) != 0 &&
+      (flags & GST_RTP_BUFFER_MAP_FLAG_SKIP_PADDING) == 0) {
     /* find memory for the padding bits */
     if (!gst_buffer_find_memory (buffer, bufsize - 1, 1, &idx, &length, &skip))
       goto wrong_length;
@@ -423,10 +424,19 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
     goto wrong_padding;
 
   rtp->buffer = buffer;
-  /* we have not yet mapped the payload */
-  rtp->data[2] = NULL;
-  rtp->size[2] = 0;
-  rtp->state = 0;
+
+  if (n_mem == 1) {
+    /* we have mapped the buffer already, so might just as well fill in the
+     * payload pointer and size and avoid another buffer map/unmap later */
+    rtp->data[2] = rtp->map[0].data + header_len;
+    rtp->size[2] = bufsize - header_len - padding;
+  } else {
+    /* we have not yet mapped the payload */
+    rtp->data[2] = NULL;
+    rtp->size[2] = 0;
+  }
+
+  /* rtp->state = 0; *//* unused */
 
   return TRUE;
 
@@ -469,7 +479,7 @@ dump_packet:
     GST_MEMDUMP ("buffer", data, size);
 
     for (i = 0; i < G_N_ELEMENTS (rtp->map); ++i) {
-      if (rtp->data[i] != NULL)
+      if (rtp->map[i].memory != NULL)
         gst_buffer_unmap (buffer, &rtp->map[i]);
     }
     return FALSE;
@@ -491,8 +501,12 @@ gst_rtp_buffer_unmap (GstRTPBuffer * rtp)
   g_return_if_fail (rtp->buffer != NULL);
 
   for (i = 0; i < 4; i++) {
-    if (rtp->data[i])
+    if (rtp->map[i].memory != NULL) {
       gst_buffer_unmap (rtp->buffer, &rtp->map[i]);
+      rtp->map[i].memory = NULL;
+    }
+    rtp->data[i] = NULL;
+    rtp->size[i] = 0;
   }
   rtp->buffer = NULL;
 }
@@ -642,7 +656,7 @@ gst_rtp_buffer_pad_to (GstRTPBuffer * rtp, guint len)
  * @rtp: the RTP packet
  *
  * Check if the extension bit is set on the RTP packet in @buffer.
- * 
+ *
  * Returns: TRUE if @buffer has the extension bit set.
  */
 gboolean
@@ -677,7 +691,7 @@ gst_rtp_buffer_set_extension (GstRTPBuffer * rtp, gboolean extension)
  *
  * If @buffer did not contain an extension, this function will return %FALSE
  * with @bits, @data and @wordlen unchanged.
- * 
+ *
  * Returns: TRUE if @buffer had the extension bit set.
  */
 gboolean
@@ -703,7 +717,7 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
 }
 
 /**
- * gst_rtp_buffer_get_extension_bytes:
+ * gst_rtp_buffer_get_extension_bytes: (rename-to gst_rtp_buffer_get_extension_data)
  * @rtp: the RTP packet
  * @bits: (out): location for header bits
  *
@@ -719,8 +733,6 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
  * Returns: (transfer full): A new #GBytes if an extension header was present
  * and %NULL otherwise.
  *
- * Rename to: gst_rtp_buffer_get_extension_data
- *
  * Since: 1.2
  */
 GBytes *
@@ -743,20 +755,45 @@ gst_rtp_buffer_get_extension_bytes (GstRTPBuffer * rtp, guint16 * bits)
   return g_bytes_new (buf_data, 4 * buf_len);
 }
 
+static gboolean
+gst_rtp_buffer_map_payload (GstRTPBuffer * rtp)
+{
+  guint hlen, plen;
+  guint idx, length;
+  gsize skip;
+
+  if (rtp->map[2].memory != NULL)
+    return TRUE;
+
+  hlen = gst_rtp_buffer_get_header_len (rtp);
+  plen = gst_buffer_get_size (rtp->buffer) - hlen - rtp->size[3];
+
+  if (!gst_buffer_find_memory (rtp->buffer, hlen, plen, &idx, &length, &skip))
+    return FALSE;
+
+  if (!gst_buffer_map_range (rtp->buffer, idx, length, &rtp->map[2],
+          rtp->map[0].flags))
+    return FALSE;
+
+  rtp->data[2] = rtp->map[2].data + skip;
+  rtp->size[2] = plen;
+
+  return TRUE;
+}
+
 /* ensure header, payload and padding are in separate buffers */
 static void
 ensure_buffers (GstRTPBuffer * rtp)
 {
   guint i, pos;
-  gsize offset;
   gboolean changed = FALSE;
 
   /* make sure payload is mapped */
-  gst_rtp_buffer_get_payload (rtp);
+  gst_rtp_buffer_map_payload (rtp);
 
   for (i = 0, pos = 0; i < 4; i++) {
     if (rtp->size[i]) {
-      offset = rtp->map[i].data - (guint8 *) rtp->data[i];
+      gsize offset = (guint8 *) rtp->data[i] - rtp->map[i].data;
 
       if (offset != 0 || rtp->map[i].size != rtp->size[i]) {
         GstMemory *mem;
@@ -774,9 +811,11 @@ ensure_buffers (GstRTPBuffer * rtp)
   }
 
   if (changed) {
+    GstBuffer *buf = rtp->buffer;
+
     gst_rtp_buffer_unmap (rtp);
-    gst_buffer_remove_memory_range (rtp->buffer, pos, -1);
-    gst_rtp_buffer_map (rtp->buffer, GST_MAP_READWRITE, rtp);
+    gst_buffer_remove_memory_range (buf, pos, -1);
+    gst_rtp_buffer_map (buf, GST_MAP_READWRITE, rtp);
   }
 }
 
@@ -850,7 +889,7 @@ gst_rtp_buffer_set_extension_data (GstRTPBuffer * rtp, guint16 bits,
  * @rtp: the RTP packet
  *
  * Get the SSRC of the RTP packet in @buffer.
- * 
+ *
  * Returns: the SSRC of @buffer in host order.
  */
 guint32
@@ -877,7 +916,7 @@ gst_rtp_buffer_set_ssrc (GstRTPBuffer * rtp, guint32 ssrc)
  * @rtp: the RTP packet
  *
  * Get the CSRC count of the RTP packet in @buffer.
- * 
+ *
  * Returns: the CSRC count of @buffer.
  */
 guint8
@@ -892,7 +931,7 @@ gst_rtp_buffer_get_csrc_count (GstRTPBuffer * rtp)
  * @idx: the index of the CSRC to get
  *
  * Get the CSRC at index @idx in @buffer.
- * 
+ *
  * Returns: the CSRC at index @idx in host order.
  */
 guint32
@@ -1124,31 +1163,17 @@ gst_rtp_buffer_get_payload_len (GstRTPBuffer * rtp)
 gpointer
 gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
 {
-  guint hlen, plen;
-  guint idx, length;
-  gsize skip;
-
   if (rtp->data[2])
     return rtp->data[2];
 
-  hlen = gst_rtp_buffer_get_header_len (rtp);
-  plen = gst_buffer_get_size (rtp->buffer) - hlen - rtp->size[3];
-
-  if (!gst_buffer_find_memory (rtp->buffer, hlen, plen, &idx, &length, &skip))
+  if (!gst_rtp_buffer_map_payload (rtp))
     return NULL;
 
-  if (!gst_buffer_map_range (rtp->buffer, idx, length, &rtp->map[2],
-          rtp->map[0].flags))
-    return NULL;
-
-  rtp->data[2] = rtp->map[2].data + skip;
-  rtp->size[2] = plen;
-
   return rtp->data[2];
 }
 
 /**
- * gst_rtp_buffer_get_payload_bytes:
+ * gst_rtp_buffer_get_payload_bytes: (rename-to gst_rtp_buffer_get_payload)
  * @rtp: the RTP packet
  *
  * Similar to gst_rtp_buffer_get_payload, but more suitable for language
@@ -1157,8 +1182,6 @@ gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
  *
  * Returns: (transfer full): A new #GBytes containing the payload data in @rtp.
  *
- * Rename to: gst_rtp_buffer_get_payload
- *
  * Since: 1.2
  */
 GBytes *
@@ -1216,6 +1239,11 @@ gst_rtp_buffer_default_clock_rate (guint8 payload_type)
 gint
 gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
 {
+  /* See http://en.wikipedia.org/wiki/Serial_number_arithmetic
+   * for an explanation why this does the right thing even for
+   * wraparounds, under the assumption that the difference is
+   * never bigger than 2**15 sequence numbers
+   */
   return (gint16) (seqnum2 - seqnum1);
 }
 
@@ -1224,18 +1252,21 @@ gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
  * @exttimestamp: a previous extended timestamp
  * @timestamp: a new timestamp
  *
- * Update the @exttimestamp field with @timestamp. For the first call of the
- * method, @exttimestamp should point to a location with a value of -1.
+ * Update the @exttimestamp field with the extended timestamp of @timestamp
+ * For the first call of the method, @exttimestamp should point to a location
+ * with a value of -1.
  *
- * This function makes sure that the returned value is a constantly increasing
- * value even in the case where there is a timestamp wraparound.
+ * This function is able to handle both forward and backward timestamps taking
+ * into account:
+ *   - timestamp wraparound making sure that the returned value is properly increased.
+ *   - timestamp unwraparound making sure that the returned value is properly decreased.
  *
- * Returns: The extended timestamp of @timestamp.
+ * Returns: The extended timestamp of @timestamp or 0 if the result can't go anywhere backwards.
  */
 guint64
 gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
 {
-  guint64 result, diff, ext;
+  guint64 result, ext;
 
   g_return_val_if_fail (exttimestamp != NULL, -1);
 
@@ -1248,17 +1279,33 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
     result = timestamp + (ext & ~(G_GUINT64_CONSTANT (0xffffffff)));
 
     /* check for timestamp wraparound */
-    if (result < ext)
-      diff = ext - result;
-    else
-      diff = result - ext;
+    if (result < ext) {
+      guint64 diff = ext - result;
 
-    if (diff > G_MAXINT32) {
-      /* timestamp went backwards more than allowed, we wrap around and get
-       * updated extended timestamp. */
-      result += (G_GUINT64_CONSTANT (1) << 32);
+      if (diff > G_MAXINT32) {
+        /* timestamp went backwards more than allowed, we wrap around and get
+         * updated extended timestamp. */
+        result += (G_GUINT64_CONSTANT (1) << 32);
+      }
+    } else {
+      guint64 diff = result - ext;
+
+      if (diff > G_MAXINT32) {
+        if (result < (G_GUINT64_CONSTANT (1) << 32)) {
+          GST_WARNING
+              ("Cannot unwrap, any wrapping took place yet. Returning 0 without updating extended timestamp.");
+          return 0;
+        } else {
+          /* timestamp went forwards more than allowed, we unwrap around and get
+           * updated extended timestamp. */
+          result -= (G_GUINT64_CONSTANT (1) << 32);
+          /* We don't want the extended timestamp storage to go back, ever */
+          return result;
+        }
+      }
     }
   }
+
   *exttimestamp = result;
 
   return result;
@@ -1475,7 +1522,7 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
 
 gboolean
 gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
-    gpointer data, guint size)
+    gconstpointer data, guint size)
 {
   guint16 bits;
   guint8 *pdata = 0;
@@ -1520,7 +1567,7 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
 
 
 static guint
-get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
+get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
 {
   guint offset = 0;
   guint bytelen = wordlen * 4;
@@ -1574,7 +1621,7 @@ get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
 
 gboolean
 gst_rtp_buffer_add_extension_twobytes_header (GstRTPBuffer * rtp,
-    guint8 appbits, guint8 id, gpointer data, guint size)
+    guint8 appbits, guint8 id, gconstpointer data, guint size)
 {
   guint16 bits;
   guint8 *pdata = 0;
