@@ -44,9 +44,10 @@
 
 /**
  * SECTION:gstrtspmessage
+ * @title: GstRTSPMessage
  * @short_description: RTSP messages
  * @see_also: gstrtspconnection
- *  
+ *
  * Provides methods for creating and parsing request, response and data messages.
  */
 
@@ -74,6 +75,40 @@ key_value_foreach (GArray * array, GFunc func, gpointer user_data)
   }
 }
 
+static void
+key_value_append (const RTSPKeyValue * kv, GArray * array)
+{
+  RTSPKeyValue kvcopy;
+  g_return_if_fail (kv != NULL);
+  g_return_if_fail (array != NULL);
+
+  kvcopy.field = kv->field;
+  kvcopy.value = g_strdup (kv->value);
+  kvcopy.custom_key = g_strdup (kv->custom_key);
+
+  g_array_append_val (array, kvcopy);
+}
+
+static GstRTSPMessage *
+gst_rtsp_message_boxed_copy (GstRTSPMessage * orig)
+{
+  GstRTSPMessage *copy;
+
+  if (gst_rtsp_message_copy (orig, &copy) == GST_RTSP_OK)
+    return copy;
+
+  return NULL;
+}
+
+static void
+gst_rtsp_message_boxed_free (GstRTSPMessage * msg)
+{
+  gst_rtsp_message_free (msg);
+}
+
+G_DEFINE_BOXED_TYPE (GstRTSPMessage, gst_rtsp_msg, gst_rtsp_message_boxed_copy,
+    gst_rtsp_message_boxed_free);
+
 /**
  * gst_rtsp_message_new:
  * @msg: (out) (transfer full): a location for the new #GstRTSPMessage
@@ -198,7 +233,7 @@ gst_rtsp_message_init_request (GstRTSPMessage * msg, GstRTSPMethod method,
  * @version: (out) (allow-none): location to hold the version
  *
  * Parse the request message @msg and store the values @method, @uri and
- * @version. The result locations can be #NULL if one is not interested in its
+ * @version. The result locations can be %NULL if one is not interested in its
  * value.
  *
  * @uri remains valid for as long as @msg is valid and unchanged.
@@ -233,9 +268,9 @@ gst_rtsp_message_parse_request (GstRTSPMessage * msg,
  * Create a new response #GstRTSPMessage with @code and @reason and store the
  * result message in @msg. Free with gst_rtsp_message_free().
  *
- * When @reason is #NULL, the default reason for @code will be used.
+ * When @reason is %NULL, the default reason for @code will be used.
  *
- * When @request is not #NULL, the relevant headers will be copied to the new
+ * When @request is not %NULL, the relevant headers will be copied to the new
  * response message.
  *
  * Returns: a #GstRTSPResult.
@@ -264,9 +299,9 @@ gst_rtsp_message_new_response (GstRTSPMessage ** msg, GstRTSPStatusCode code,
  *
  * Initialize @msg with @code and @reason.
  *
- * When @reason is #NULL, the default reason for @code will be used.
+ * When @reason is %NULL, the default reason for @code will be used.
  *
- * When @request is not #NULL, the relevant headers will be copied to the new
+ * When @request is not %NULL, the relevant headers will be copied to the new
  * response message.
  *
  * Returns: a #GstRTSPResult.
@@ -332,7 +367,7 @@ gst_rtsp_message_init_response (GstRTSPMessage * msg, GstRTSPStatusCode code,
  * @version: (out) (allow-none): location to hold the version
  *
  * Parse the response message @msg and store the values @code, @reason and
- * @version. The result locations can be #NULL if one is not interested in its
+ * @version. The result locations can be %NULL if one is not interested in its
  * value.
  *
  * @reason remains valid for as long as @msg is valid and unchanged.
@@ -429,7 +464,7 @@ gst_rtsp_message_parse_data (GstRTSPMessage * msg, guint8 * channel)
  * @msg: a #GstRTSPMessage
  *
  * Unset the contents of @msg so that it becomes an uninitialized
- * #GstRTSPMessage again. This function is mostly used in combination with 
+ * #GstRTSPMessage again. This function is mostly used in combination with
  * gst_rtsp_message_init_request(), gst_rtsp_message_init_response() and
  * gst_rtsp_message_init_data() on stack allocated #GstRTSPMessage structures.
  *
@@ -497,6 +532,63 @@ gst_rtsp_message_free (GstRTSPMessage * msg)
   return res;
 }
 
+/**
+ * gst_rtsp_message_copy:
+ * @msg: a #GstRTSPMessage
+ * @copy: (out) (transfer full): pointer to new #GstRTSPMessage
+ *
+ * Allocate a new copy of @msg and store the result in @copy. The value in
+ * @copy should be release with gst_rtsp_message_free function.
+ *
+ * Returns: a #GstRTSPResult
+ *
+ * Since: 1.14
+ */
+GstRTSPResult
+gst_rtsp_message_copy (const GstRTSPMessage * msg, GstRTSPMessage ** copy)
+{
+  GstRTSPResult ret;
+  GstRTSPMessage *cp;
+
+  if (msg == NULL)
+    return GST_RTSP_EINVAL;
+
+  ret = gst_rtsp_message_new (copy);
+  if (ret != GST_RTSP_OK)
+    return ret;
+
+  cp = *copy;
+
+  cp->type = msg->type;
+  switch (cp->type) {
+    case GST_RTSP_MESSAGE_INVALID:
+      break;
+    case GST_RTSP_MESSAGE_REQUEST:
+    case GST_RTSP_MESSAGE_HTTP_REQUEST:
+      cp->type_data.request.method = msg->type_data.request.method;
+      cp->type_data.request.uri = g_strdup (msg->type_data.request.uri);
+      cp->type_data.request.version = msg->type_data.request.version;
+      break;
+    case GST_RTSP_MESSAGE_RESPONSE:
+    case GST_RTSP_MESSAGE_HTTP_RESPONSE:
+      cp->type_data.response.code = msg->type_data.response.code;
+      cp->type_data.response.reason = g_strdup (msg->type_data.response.reason);
+      cp->type_data.response.version = msg->type_data.response.version;
+      break;
+    case GST_RTSP_MESSAGE_DATA:
+      cp->type_data.data.channel = msg->type_data.data.channel;
+      break;
+    default:
+      return GST_RTSP_EINVAL;
+  }
+
+  key_value_foreach (msg->hdr_fields, (GFunc) key_value_append, cp->hdr_fields);
+  gst_rtsp_message_set_body (cp, msg->body, msg->body_size);
+
+  return GST_RTSP_OK;
+}
+
+
 /**
  * gst_rtsp_message_take_header:
  * @msg: a #GstRTSPMessage
@@ -903,7 +995,7 @@ gst_rtsp_message_get_body (const GstRTSPMessage * msg, guint8 ** data,
  * @size: (out): location for the size of @data
  *
  * Take the body of @msg and store it in @data and @size. After this method,
- * the body and size of @msg will be set to #NULL and 0 respectively.
+ * the body and size of @msg will be set to %NULL and 0 respectively.
  *
  * Returns: #GST_RTSP_OK.
  */
@@ -1021,3 +1113,287 @@ gst_rtsp_message_dump (GstRTSPMessage * msg)
   }
   return GST_RTSP_OK;
 }
+
+
+static const gchar *
+skip_lws (const gchar * s)
+{
+  while (g_ascii_isspace (*s))
+    s++;
+  return s;
+}
+
+static const gchar *
+skip_commas (const gchar * s)
+{
+  /* The grammar allows for multiple commas */
+  while (g_ascii_isspace (*s) || *s == ',')
+    s++;
+  return s;
+}
+
+static const gchar *
+skip_scheme (const gchar * s)
+{
+  while (*s && !g_ascii_isspace (*s))
+    s++;
+  return s;
+}
+
+static const gchar *
+skip_item (const gchar * s)
+{
+  gboolean quoted = FALSE;
+
+  /* A list item ends at the last non-whitespace character
+   * before a comma which is not inside a quoted-string. Or at
+   * the end of the string.
+   */
+  while (*s) {
+    if (*s == '"') {
+      quoted = !quoted;
+    } else if (quoted) {
+      if (*s == '\\' && *(s + 1))
+        s++;
+    } else {
+      if (*s == ',' || g_ascii_isspace (*s))
+        break;
+    }
+    s++;
+  }
+
+  return s;
+}
+
+static void
+decode_quoted_string (gchar * quoted_string)
+{
+  gchar *src, *dst;
+
+  src = quoted_string + 1;
+  dst = quoted_string;
+  while (*src && *src != '"') {
+    if (*src == '\\' && *(src + 1))
+      src++;
+    *dst++ = *src++;
+  }
+  *dst = '\0';
+}
+
+static void
+parse_auth_credentials (GPtrArray * auth_credentials, const gchar * header,
+    GstRTSPHeaderField field)
+{
+  while (header[0] != '\0') {
+    const gchar *end;
+    GstRTSPAuthCredential *auth_credential;
+
+    /* Skip whitespace at the start of the string */
+    header = skip_lws (header);
+    if (header[0] == '\0')
+      break;
+
+    /* Skip until end of string or whitespace: end of scheme */
+    end = skip_scheme (header);
+
+    auth_credential = g_new0 (GstRTSPAuthCredential, 1);
+
+    if (g_ascii_strncasecmp (header, "basic", 5) == 0) {
+      auth_credential->scheme = GST_RTSP_AUTH_BASIC;
+    } else if (g_ascii_strncasecmp (header, "digest", 6) == 0) {
+      auth_credential->scheme = GST_RTSP_AUTH_DIGEST;
+    } else {
+      /* Not supported, skip */
+      g_free (auth_credential);
+      header = end;
+      continue;
+    }
+
+    /* Basic Authorization request has only an unformated blurb following, all
+     * other variants have comma-separated name=value pairs */
+    if (end[0] != '\0' && field == GST_RTSP_HDR_AUTHORIZATION
+        && auth_credential->scheme == GST_RTSP_AUTH_BASIC) {
+      auth_credential->authorization = g_strdup (end + 1);
+      header = end;
+    } else if (end[0] != '\0') {
+      GPtrArray *params;
+
+      params = g_ptr_array_new ();
+
+      /* Space or start of param */
+      header = end;
+
+      /* Parse a header whose content is described by RFC2616 as
+       * "#something", where "something" does not itself contain commas,
+       * except as part of quoted-strings, into a list of allocated strings.
+       */
+      while (*header) {
+        const gchar *item_end;
+        const gchar *eq;
+
+        header = skip_commas (header);
+        item_end = skip_item (header);
+
+        for (eq = header; *eq != '\0' && *eq != '=' && eq < item_end; eq++);
+        if (eq[0] == '=') {
+          GstRTSPAuthParam *auth_param = g_new0 (GstRTSPAuthParam, 1);
+          const gchar *value;
+
+          /* have an actual param */
+          auth_param->name = g_strndup (header, eq - header);
+
+          value = eq + 1;
+          value = skip_lws (value);
+          auth_param->value = g_strndup (value, item_end - value);
+          if (value[0] == '"')
+            decode_quoted_string (auth_param->value);
+
+          g_ptr_array_add (params, auth_param);
+          header = item_end;
+        } else {
+          /* at next scheme, header at start of it */
+          break;
+        }
+      }
+      if (params->len)
+        g_ptr_array_add (params, NULL);
+      auth_credential->params =
+          (GstRTSPAuthParam **) g_ptr_array_free (params, FALSE);
+    } else {
+      header = end;
+    }
+    g_ptr_array_add (auth_credentials, auth_credential);
+
+    /* WWW-Authenticate allows multiple, Authorization allows one */
+    if (field == GST_RTSP_HDR_AUTHORIZATION)
+      break;
+  }
+}
+
+/**
+ * gst_rtsp_message_parse_auth_credentials:
+ * @msg: a #GstRTSPMessage
+ * @field: a #GstRTSPHeaderField
+ *
+ * Parses the credentials given in a WWW-Authenticate or Authorization header.
+ *
+ * Returns: %NULL-terminated array of GstRTSPAuthCredential or %NULL.
+ *
+ * Since: 1.12
+ */
+GstRTSPAuthCredential **
+gst_rtsp_message_parse_auth_credentials (GstRTSPMessage * msg,
+    GstRTSPHeaderField field)
+{
+  gchar *header;
+  GPtrArray *auth_credentials;
+  gint i;
+
+  g_return_val_if_fail (msg != NULL, NULL);
+
+  auth_credentials = g_ptr_array_new ();
+
+  i = 0;
+  while (gst_rtsp_message_get_header (msg, field, &header, i) == GST_RTSP_OK) {
+    parse_auth_credentials (auth_credentials, header, field);
+    i++;
+  }
+
+  if (auth_credentials->len)
+    g_ptr_array_add (auth_credentials, NULL);
+
+  return (GstRTSPAuthCredential **) g_ptr_array_free (auth_credentials, FALSE);
+}
+
+GstRTSPAuthParam *
+gst_rtsp_auth_param_copy (GstRTSPAuthParam * param)
+{
+  GstRTSPAuthParam *copy;
+
+  if (param == NULL)
+    return NULL;
+
+  copy = g_new0 (GstRTSPAuthParam, 1);
+  copy->name = g_strdup (param->name);
+  copy->value = g_strdup (param->value);
+
+  return copy;
+}
+
+void
+gst_rtsp_auth_param_free (GstRTSPAuthParam * param)
+{
+  if (param != NULL) {
+    g_free (param->name);
+    g_free (param->value);
+    g_free (param);
+  }
+}
+
+G_DEFINE_BOXED_TYPE (GstRTSPAuthParam, gst_rtsp_auth_param,
+    (GBoxedCopyFunc) gst_rtsp_auth_param_copy,
+    (GBoxedFreeFunc) gst_rtsp_auth_param_free);
+
+static void
+gst_rtsp_auth_credential_free (GstRTSPAuthCredential * credential)
+{
+  GstRTSPAuthParam **p;
+
+  if (credential == NULL)
+    return;
+
+  for (p = credential->params; p != NULL && *p != NULL; ++p)
+    gst_rtsp_auth_param_free (*p);
+
+  g_free (credential->params);
+  g_free (credential->authorization);
+  g_free (credential);
+}
+
+static GstRTSPAuthCredential *
+gst_rtsp_auth_credential_copy (GstRTSPAuthCredential * cred)
+{
+  GstRTSPAuthCredential *copy;
+
+  if (cred == NULL)
+    return NULL;
+
+  copy = g_new0 (GstRTSPAuthCredential, 1);
+  copy->scheme = cred->scheme;
+  if (cred->params) {
+    guint i, n_params = g_strv_length ((gchar **) cred->params);
+
+    copy->params = g_new0 (GstRTSPAuthParam *, n_params + 1);
+    for (i = 0; i < n_params; ++i)
+      copy->params[i] = gst_rtsp_auth_param_copy (cred->params[i]);
+  }
+  copy->authorization = g_strdup (cred->authorization);
+  return copy;
+}
+
+/**
+ * gst_rtsp_auth_credentials_free:
+ * @credentials: a %NULL-terminated array of #GstRTSPAuthCredential
+ *
+ * Free a %NULL-terminated array of credentials returned from
+ * gst_rtsp_message_parse_auth_credentials().
+ *
+ * Since: 1.12
+ */
+void
+gst_rtsp_auth_credentials_free (GstRTSPAuthCredential ** credentials)
+{
+  GstRTSPAuthCredential **p;
+
+  if (!credentials)
+    return;
+
+  for (p = credentials; p != NULL && *p != NULL; ++p)
+    gst_rtsp_auth_credential_free (*p);
+
+  g_free (credentials);
+}
+
+G_DEFINE_BOXED_TYPE (GstRTSPAuthCredential, gst_rtsp_auth_credential,
+    (GBoxedCopyFunc) gst_rtsp_auth_credential_copy,
+    (GBoxedFreeFunc) gst_rtsp_auth_credential_free);
