@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -23,61 +23,81 @@
 #define __GST_TAG_TAG_H__
 
 #include <gst/gst.h>
+#include <gst/tag/tag-prelude.h>
+#include <gst/tag/gsttagdemux.h>
+#include <gst/tag/gsttagmux.h>
+#include <gst/tag/xmpwriter.h>
+#include <gst/tag/tag-enumtypes.h>
 
 G_BEGIN_DECLS
 
 /* Tag names */
 
 /**
- * GST_TAG_MUSICBRAINZ_TRACKID
+ * GST_TAG_MUSICBRAINZ_TRACKID:
  *
  * MusicBrainz track ID
  */
 #define GST_TAG_MUSICBRAINZ_TRACKID	"musicbrainz-trackid"
 /**
- * GST_TAG_MUSICBRAINZ_ARTISTID
+ * GST_TAG_MUSICBRAINZ_ARTISTID:
  *
  * MusicBrainz artist ID
  */
 #define GST_TAG_MUSICBRAINZ_ARTISTID	"musicbrainz-artistid"
 /**
- * GST_TAG_MUSICBRAINZ_ALBUMID
+ * GST_TAG_MUSICBRAINZ_ALBUMID:
  *
  * MusicBrainz album ID
  */
 #define GST_TAG_MUSICBRAINZ_ALBUMID	"musicbrainz-albumid"
 /**
- * GST_TAG_MUSICBRAINZ_ALBUMARTISTID
+ * GST_TAG_MUSICBRAINZ_ALBUMARTISTID:
  *
  * MusicBrainz album artist ID
  */
 #define GST_TAG_MUSICBRAINZ_ALBUMARTISTID	"musicbrainz-albumartistid"
 /**
- * GST_TAG_MUSICBRAINZ_TRMID
+ * GST_TAG_MUSICBRAINZ_TRMID:
  *
  * MusicBrainz track TRM ID
  */
 #define GST_TAG_MUSICBRAINZ_TRMID	"musicbrainz-trmid"
 
 /**
- * GST_TAG_CMML_STREAM
+ * GST_TAG_CMML_STREAM:
  *
  * Annodex CMML stream element tag
  */
 #define GST_TAG_CMML_STREAM "cmml-stream"
 /**
- * GST_TAG_CMML_HEAD
+ * GST_TAG_CMML_HEAD:
  *
  * Annodex CMML head element tag
  */
 
 #define GST_TAG_CMML_HEAD "cmml-head"
 /**
- * GST_TAG_CMML_CLIP
+ * GST_TAG_CMML_CLIP:
  *
  * Annodex CMML clip element tag
  */
 #define GST_TAG_CMML_CLIP "cmml-clip"
+/**
+ * GST_TAG_MUSICAL_KEY:
+ *
+ * Musical key in which the sound starts. It is represented as a string
+ * with a maximum length of three characters. The ground keys are
+ * represented with "A","B","C","D","E", "F" and "G" and halfkeys
+ * represented with "b" and "#". Minor is represented as "m" (e.g. "Dbm").
+ * Off key is represented with an "o" only.
+ * This notation might be extended in the future to support non-minor/major
+ * keys.
+ *
+ * Since: 1.2
+ */
+#define GST_TAG_MUSICAL_KEY                   "musical-key"
+
 
 /* CDDA tags */
 
@@ -113,8 +133,6 @@ G_BEGIN_DECLS
  * GST_TAG_CAPTURING_SHUTTER_SPEED:
  *
  * Shutter speed used when capturing an image, in seconds. (fraction)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_SHUTTER_SPEED        "capturing-shutter-speed"
 
@@ -125,8 +143,6 @@ G_BEGIN_DECLS
  *
  * The value stored is the denominator of the focal ratio (f-number).
  * For example, if this tag value is 2, the focal ratio is f/2.
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_FOCAL_RATIO          "capturing-focal-ratio"
 
@@ -134,17 +150,22 @@ G_BEGIN_DECLS
  * GST_TAG_CAPTURING_FOCAL_LENGTH:
  *
  * Focal length used when capturing an image, in mm. (double)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_FOCAL_LENGTH         "capturing-focal-length"
 
+/**
+ * GST_TAG_CAPTURING_FOCAL_LENGTH_35_MM:
+ *
+ * 35 mm equivalent focal length used when capturing an image, in mm. (double)
+ *
+ * Since: 1.10
+ */
+#define GST_TAG_CAPTURING_FOCAL_LENGTH_35_MM   "capturing-focal-length-35mm"
+
 /**
  * GST_TAG_CAPTURING_DIGITAL_ZOOM_RATIO:
  *
  * Digital zoom ratio used when capturing an image. (double)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_DIGITAL_ZOOM_RATIO   "capturing-digital-zoom-ratio"
 
@@ -152,8 +173,6 @@ G_BEGIN_DECLS
  * GST_TAG_CAPTURING_ISO_SPEED:
  *
  * ISO speed used when capturing an image. (integer)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_ISO_SPEED           "capturing-iso-speed"
 
@@ -172,8 +191,6 @@ G_BEGIN_DECLS
  *   "action" - biased towards fast shutter speed
  *   "portrait" - closeup, leaving background out of focus
  *   "landscape" - landscape photos, background in focus
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_EXPOSURE_PROGRAM     "capturing-exposure-program"
 
@@ -186,8 +203,6 @@ G_BEGIN_DECLS
  *   "auto-exposure"
  *   "manual-exposure"
  *   "auto-bracket"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_EXPOSURE_MODE       "capturing-exposure-mode"
 
@@ -195,8 +210,6 @@ G_BEGIN_DECLS
  * GST_TAG_CAPTURING_EXPOSURE_COMPENSATION:
  *
  * Exposure compensation using when capturing an image in EV. (double)
- *
- * Since: 0.10.33
  */
 #define GST_TAG_CAPTURING_EXPOSURE_COMPENSATION "capturing-exposure-compensation"
 
@@ -210,8 +223,6 @@ G_BEGIN_DECLS
  *   "landscape"
  *   "portrait"
  *   "night-scene"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_SCENE_CAPTURE_TYPE  "capturing-scene-capture-type"
 
@@ -226,8 +237,6 @@ G_BEGIN_DECLS
  *   "high-gain-up"
  *   "low-gain-down"
  *   "high-gain-down"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_GAIN_ADJUSTMENT     "capturing-gain-adjustment"
 
@@ -245,8 +254,6 @@ G_BEGIN_DECLS
  *   "fluorescent"
  *   "fluorescent h" (newer daylight-calibrated fluorescents)
  *   "flash"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_WHITE_BALANCE       "capturing-white-balance"
 
@@ -259,8 +266,6 @@ G_BEGIN_DECLS
  *  "normal"
  *  "soft"
  *  "hard"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_CONTRAST            "capturing-contrast"
 
@@ -273,8 +278,6 @@ G_BEGIN_DECLS
  *  "normal"
  *  "low-saturation"
  *  "high-saturation"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_SATURATION          "capturing-saturation"
 
@@ -287,8 +290,6 @@ G_BEGIN_DECLS
  *  "normal"
  *  "soft"
  *  "hard"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_SHARPNESS          "capturing-sharpness"
 
@@ -299,8 +300,6 @@ G_BEGIN_DECLS
  *
  * Note that if this tag isn't present, it should not be assumed that
  * the flash did not fire. It should be treated as unknown.
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_FLASH_FIRED         "capturing-flash-fired"
 
@@ -313,8 +312,6 @@ G_BEGIN_DECLS
  *  "auto"
  *  "always"
  *  "never"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_FLASH_MODE         "capturing-flash-mode"
 
@@ -332,8 +329,6 @@ G_BEGIN_DECLS
  *   "pattern"
  *   "partial"
  *   "other"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_METERING_MODE      "capturing-metering-mode"
 
@@ -348,8 +343,6 @@ G_BEGIN_DECLS
  *   "transparent-scanner"
  *   "reflex-scanner"
  *   "other"
- *
- * Since: 0.10.31
  */
 #define GST_TAG_CAPTURING_SOURCE             "capturing-source"
 
@@ -357,16 +350,12 @@ G_BEGIN_DECLS
  * GST_TAG_IMAGE_HORIZONTAL_PPI:
  *
  * Media (image/video) intended horizontal pixel density in ppi. (double)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_IMAGE_HORIZONTAL_PPI         "image-horizontal-ppi"
 /**
  * GST_TAG_IMAGE_VERTICAL_PPI:
  *
  * Media (image/video) intended vertical pixel density in ppi. (double)
- *
- * Since: 0.10.31
  */
 #define GST_TAG_IMAGE_VERTICAL_PPI           "image-vertical-ppi"
 
@@ -377,7 +366,7 @@ G_BEGIN_DECLS
  * GstTagImageType:
  * @GST_TAG_IMAGE_TYPE_NONE                  : No image type. Can be used to
  *      tell functions such as gst_tag_image_data_to_image_sample() that no
- *      image type should be set. (Since: 0.10.20)
+ *      image type should be set.
  * @GST_TAG_IMAGE_TYPE_UNDEFINED             : Undefined/other image type
  * @GST_TAG_IMAGE_TYPE_FRONT_COVER           : Cover (front)
  * @GST_TAG_IMAGE_TYPE_BACK_COVER            : Cover (back)
@@ -398,10 +387,8 @@ G_BEGIN_DECLS
  * @GST_TAG_IMAGE_TYPE_BAND_ARTIST_LOGO      : Band/artist logotype
  * @GST_TAG_IMAGE_TYPE_PUBLISHER_STUDIO_LOGO : Publisher/studio logotype
  *
- * Type of image contained in an image tag (specified as field in
- * the image buffer's caps structure)
- *
- * Since: 0.10.9
+ * Type of image contained in an image tag (specified as "image-type" field in
+ * the info structure in the image's #GstSample)
  */
 /* Note: keep in sync with register_tag_image_type_enum() */
 typedef enum {
@@ -427,40 +414,47 @@ typedef enum {
   GST_TAG_IMAGE_TYPE_PUBLISHER_STUDIO_LOGO
 } GstTagImageType;
 
-#define GST_TYPE_TAG_IMAGE_TYPE  (gst_tag_image_type_get_type ())
-GType   gst_tag_image_type_get_type (void);
-
 /**
  * GST_TAG_ID3V2_HEADER_SIZE:
  *
  * ID3V2 header size considered minimum input for some functions such as
  * gst_tag_list_from_id3v2_tag() and gst_tag_get_id3v2_tag_size() for example.
- *
- * Since: 0.10.36
  */
 #define GST_TAG_ID3V2_HEADER_SIZE            10
 
 /* functions for vorbis comment manipulation */
 
+GST_TAG_API
 const gchar *           gst_tag_from_vorbis_tag                 (const gchar *          vorbis_tag);
+
+GST_TAG_API
 const gchar *           gst_tag_to_vorbis_tag                   (const gchar *          gst_tag);
+
+GST_TAG_API
 void                    gst_vorbis_tag_add                      (GstTagList *           list, 
                                                                  const gchar *          tag, 
                                                                  const gchar *          value);
 
+GST_TAG_API
 GList *                 gst_tag_to_vorbis_comments              (const GstTagList *     list, 
                                                                  const gchar *          tag);
 
 /* functions to convert GstBuffers with vorbiscomment contents to GstTagLists and back */
+
+GST_TAG_API
 GstTagList *            gst_tag_list_from_vorbiscomment         (const guint8 *         data,
                                                                  gsize                  size,
                                                                  const guint8 *         id_data,
                                                                  const guint            id_data_length,
                                                                  gchar **               vendor_string);
+
+GST_TAG_API
 GstTagList *            gst_tag_list_from_vorbiscomment_buffer  (GstBuffer *            buffer,
                                                                  const guint8 *         id_data,
                                                                  const guint            id_data_length,
                                                                  gchar **               vendor_string);
+
+GST_TAG_API
 GstBuffer *             gst_tag_list_to_vorbiscomment_buffer    (const GstTagList *     list,
                                                                  const guint8 *         id_data,
                                                                  const guint            id_data_length,
@@ -472,78 +466,112 @@ GstBuffer *             gst_tag_list_to_vorbiscomment_buffer    (const GstTagLis
  * gst_tag_list_from_id3v2_tag(). Also, note gst.tag.list_xyz() namespace vs. gst.tag_list_xyz(),
  * which is a bit confusing and possibly doesn't map too well */
 
+GST_TAG_API
 guint                   gst_tag_id3_genre_count                 (void);
+
+GST_TAG_API
 const gchar *           gst_tag_id3_genre_get                   (const guint            id);
+
+GST_TAG_API
 GstTagList *            gst_tag_list_new_from_id3v1             (const guint8 *         data);
 
+GST_TAG_API
 const gchar *           gst_tag_from_id3_tag                    (const gchar *          id3_tag);
+
+GST_TAG_API
 const gchar *           gst_tag_from_id3_user_tag               (const gchar *          type,
                                                                  const gchar *          id3_user_tag);
+
+GST_TAG_API
 const gchar *           gst_tag_to_id3_tag                      (const gchar *          gst_tag);
 
+GST_TAG_API
 gboolean                gst_tag_list_add_id3_image (GstTagList   * tag_list,
                                                     const guint8 * image_data,
                                                     guint          image_data_len,
                                                     guint          id3_picture_type);
 
+GST_TAG_API
 GstTagList *            gst_tag_list_from_id3v2_tag (GstBuffer * buffer);
 
+GST_TAG_API
 guint                   gst_tag_get_id3v2_tag_size  (GstBuffer * buffer);
 
 /* functions to  convert GstBuffers with xmp packets contents to GstTagLists and back */
-GstTagList *            gst_tag_list_from_xmp_buffer  (GstBuffer *  buffer);
-GstBuffer *             gst_tag_list_to_xmp_buffer    (const GstTagList * list,
-                                                       gboolean           read_only);
-GstBuffer *		gst_tag_list_to_xmp_buffer_full (const GstTagList * list,
-							gboolean read_only, const gchar ** schemas);
-const gchar**		gst_tag_xmp_list_schemas      (void);
+
+GST_TAG_API
+GstTagList *            gst_tag_list_from_xmp_buffer (GstBuffer *  buffer);
+
+GST_TAG_API
+GstBuffer *		          gst_tag_list_to_xmp_buffer   (const GstTagList * list,
+							                                        gboolean read_only,
+							                                        const gchar ** schemas);
+
+GST_TAG_API
+const gchar**		        gst_tag_xmp_list_schemas     (void);
 
 /* functions related to exif */
+
+GST_TAG_API
 GstBuffer *             gst_tag_list_to_exif_buffer (const GstTagList * taglist,
                                                      gint byte_order,
                                                      guint32 base_offset);
 
+GST_TAG_API
 GstBuffer *             gst_tag_list_to_exif_buffer_with_tiff_header (const GstTagList * taglist);
 
+GST_TAG_API
 GstTagList *            gst_tag_list_from_exif_buffer (GstBuffer * buffer,
                                                        gint byte_order,
                                                        guint32 base_offset);
 
+GST_TAG_API
 GstTagList *            gst_tag_list_from_exif_buffer_with_tiff_header (
                                                       GstBuffer * buffer);
 
 /* other tag-related functions */
 
+GST_TAG_API
 gboolean                gst_tag_parse_extended_comment (const gchar  * ext_comment,
                                                         gchar       ** key,
                                                         gchar       ** lang,
                                                         gchar       ** value,
                                                         gboolean       fail_if_no_key);
 
+GST_TAG_API
 gchar                 * gst_tag_freeform_string_to_utf8 (const gchar  * data,
                                                          gint           size,
                                                          const gchar ** env_vars);
 
+GST_TAG_API
 GstSample             * gst_tag_image_data_to_image_sample (const guint8   * image_data,
                                                             guint            image_data_len,
                                                             GstTagImageType  image_type);
 
 /* FIXME 0.11: get rid of this awkward register/init function, see tags.c */
+
+GST_TAG_API
 void                    gst_tag_register_musicbrainz_tags (void);
 
 
 /* language tag related functions */
 
+GST_TAG_API
 gchar **       gst_tag_get_language_codes (void);
 
+GST_TAG_API
 const gchar *  gst_tag_get_language_name (const gchar * language_code);
 
+GST_TAG_API
 const gchar *  gst_tag_get_language_code_iso_639_1 (const gchar * lang_code);
 
+GST_TAG_API
 const gchar *  gst_tag_get_language_code_iso_639_2B (const gchar * lang_code);
 
+GST_TAG_API
 const gchar *  gst_tag_get_language_code_iso_639_2T (const gchar * lang_code);
 
+GST_TAG_API
 gboolean       gst_tag_check_language_code          (const gchar * lang_code);
 
 /**
@@ -551,8 +579,6 @@ gboolean       gst_tag_check_language_code          (const gchar * lang_code);
  * @lang_code: ISO-639 language code (e.g. "deu" or "ger" or "de")
  *
  * Convenience macro wrapping gst_tag_get_language_code_iso_639_1().
- *
- * Since: 0.10.26
  */
 #define gst_tag_get_language_code(lang_code) \
     gst_tag_get_language_code_iso_639_1(lang_code)
@@ -596,8 +622,6 @@ gboolean       gst_tag_check_language_code          (const gchar * lang_code);
  *     created by the Free Software Foundation (FSF)
  *
  * See http://creativecommons.org/ns for more information.
- *
- * Since: 0.10.36
  */
 typedef enum {
   GST_TAG_LICENSE_PERMITS_REPRODUCTION             = (1 << 0),
@@ -619,22 +643,27 @@ typedef enum {
   GST_TAG_LICENSE_FREE_SOFTWARE_FOUNDATION_LICENSE = (1 << 25)
 } GstTagLicenseFlags;
 
+GST_TAG_API
 gchar **            gst_tag_get_licenses (void);
 
+GST_TAG_API
 GstTagLicenseFlags  gst_tag_get_license_flags (const gchar * license_ref);
 
+GST_TAG_API
 const gchar *       gst_tag_get_license_nick  (const gchar * license_ref);
 
+GST_TAG_API
 const gchar *       gst_tag_get_license_title (const gchar * license_ref);
 
+GST_TAG_API
 const gchar *       gst_tag_get_license_version (const gchar * license_ref);
 
+GST_TAG_API
 const gchar *       gst_tag_get_license_description  (const gchar * license_ref);
 
+GST_TAG_API
 const gchar *       gst_tag_get_license_jurisdiction (const gchar * license_ref);
 
-GType               gst_tag_license_flags_get_type (void);
-
 G_END_DECLS
 
 #endif /* __GST_TAG_TAG_H__ */
