@@ -36,6 +36,7 @@
 
 /**
  * SECTION:element-decodebin3
+ * @title: decodebin3
  *
  * #GstBin that auto-magically constructs a decoding pipeline using available
  * decoders and demuxers via auto-plugging. The output is raw audio, video
@@ -43,27 +44,20 @@
  *
  * decodebin3 differs from the previous decodebin (decodebin2) in important ways:
  *
- * <itemizedlist>
- * <listitem>
- * supports publication and selection of stream information via
+ * * supports publication and selection of stream information via
  * GstStreamCollection messages and #GST_EVENT_SELECT_STREAM events.
- * </listitem>
- * <listitem>
- * dynamically switches stream connections internally, and
+ *
+ * * dynamically switches stream connections internally, and
  * reuses decoder elements when stream selections change, so that in
  * the normal case it maintains 1 decoder of each type (video/audio/subtitle)
  * and only creates new elements when streams change and an existing decoder
  * is not capable of handling the new format.
- * </listitem>
- * <listitem>
- * supports multiple input pads for the parallel decoding of auxilliary streams
+ *
+ * * supports multiple input pads for the parallel decoding of auxilliary streams
  * not muxed with the primary stream.
- * </listitem>
- * <listitem>
- * does not handle network stream buffering. decodebin3 expects that network stream
+ *
+ * * does not handle network stream buffering. decodebin3 expects that network stream
  * buffering is handled upstream, before data is passed to it.
- * </listitem>
- * </itemizedlist>
  *
  * <emphasis>decodebin3 is still experimental API and a technology preview.
  * Its behaviour and exposed API is subject to change.</emphasis>
@@ -74,7 +68,7 @@
  * Global design
  *
  * 1) From sink pad to elementary streams (GstParseBin)
- * 
+ *
  * The input sink pads are fed to GstParseBin. GstParseBin will feed them
  * through typefind. When the caps are detected (or changed) we recursively
  * figure out which demuxer, parser or depayloader is needed until we get to
@@ -115,7 +109,6 @@
  *  * STREAM_START :
  *     a new stream is starting => link it further if needed
  *
- *
  * 3) Gradual replacement
  *
  * If the caps change at any point in decodebin (input sink pad, demuxer output,
@@ -133,8 +126,6 @@
  *  b.1) The new CAPS are accepted, keep current configuration
  *  b.2) The new CAPS are not accepted, remove following elements then do a)
  *
- *
- *
  *    Components:
  *
  *                                                   MultiQ     Output
@@ -209,23 +200,22 @@ struct _GstDecodebin3
   GList *other_inputs;
   /* counter for input */
   guint32 input_counter;
-  /* Current stream group_id (default : G_MAXUINT32) */
+  /* Current stream group_id (default : GST_GROUP_ID_INVALID) */
   /* FIXME : Needs to be resetted appropriately (when upstream changes ?) */
   guint32 current_group_id;
   /* End of variables protected by input_lock */
 
   GstElement *multiqueue;
 
-  /* FIXME : Mutex for protecting values below */
-  GstStreamCollection *collection;      /* Active collection */
-
+  /* selection_lock protects access to following variables */
+  GMutex selection_lock;
   GList *input_streams;         /* List of DecodebinInputStream for active collection */
   GList *output_streams;        /* List of DecodebinOutputStream used for output */
   GList *slots;                 /* List of MultiQueueSlot */
   guint slot_id;
 
-  /* selection_lock protects access to following variables */
-  GMutex selection_lock;
+  /* Active collection */
+  GstStreamCollection *collection;
   /* requested selection of stream-id to activate post-multiqueue */
   GList *requested_selection;
   /* list of stream-id currently activated in output */
@@ -245,7 +235,6 @@ struct _GstDecodebin3
    * FIXME : Is this really needed ? */
   GList *pending_collection;
 
-
   /* Factories */
   GMutex factories_lock;
   guint32 factories_cookie;
@@ -289,6 +278,12 @@ struct _DecodebinInput
 
   gulong pad_added_sigid;
   gulong pad_removed_sigid;
+  gulong drained_sigid;
+
+  /* TRUE if the input got drained
+   * FIXME : When do we reset it if re-used ?
+   */
+  gboolean drained;
 
   /* HACK : Remove these fields */
   /* List of PendingPad structures */
@@ -317,7 +312,7 @@ typedef struct _MultiQueueSlot
   /* id of the MQ src_pad event probe */
   gulong probe_id;
 
-  gboolean drain_eos;
+  gboolean is_drained;
 
   DecodebinOutputStream *output;
 } MultiQueueSlot;
@@ -358,8 +353,6 @@ typedef struct _PendingPad
 } PendingPad;
 
 /* properties */
-#define DEFAULT_CAPS (gst_static_caps_get (&default_raw_caps))
-
 enum
 {
   PROP_0,
@@ -370,6 +363,7 @@ enum
 enum
 {
   SIGNAL_SELECT_STREAM,
+  SIGNAL_ABOUT_TO_FINISH,
   LAST_SIGNAL
 };
 static guint gst_decodebin3_signals[LAST_SIGNAL] = { 0 };
@@ -482,6 +476,7 @@ static gboolean have_factory (GstDecodebin3 * dbin, GstCaps * caps,
 #endif
 
 static void free_input (GstDecodebin3 * dbin, DecodebinInput * input);
+static void free_input_async (GstDecodebin3 * dbin, DecodebinInput * input);
 static DecodebinInput *create_new_input (GstDecodebin3 * dbin, gboolean main);
 static gboolean set_input_group_id (DecodebinInput * input, guint32 * group_id);
 
@@ -500,6 +495,11 @@ static MultiQueueSlot *get_slot_for_input (GstDecodebin3 * dbin,
 static void link_input_to_slot (DecodebinInputStream * input,
     MultiQueueSlot * slot);
 static void free_multiqueue_slot (GstDecodebin3 * dbin, MultiQueueSlot * slot);
+static void free_multiqueue_slot_async (GstDecodebin3 * dbin,
+    MultiQueueSlot * slot);
+
+static GstStreamCollection *get_merged_collection (GstDecodebin3 * dbin);
+static void update_requested_selection (GstDecodebin3 * dbin);
 
 /* FIXME: Really make all the parser stuff a self-contained helper object */
 #include "gstdecodebin3-parse.c"
@@ -555,6 +555,17 @@ gst_decodebin3_class_init (GstDecodebin3Class * klass)
       _gst_int_accumulator, NULL, g_cclosure_marshal_generic,
       G_TYPE_INT, 2, GST_TYPE_STREAM_COLLECTION, GST_TYPE_STREAM);
 
+  /**
+   * GstDecodebin3::about-to-finish:
+   *
+   * This signal is emitted when the data for the selected URI is
+   * entirely buffered and it is safe to specify anothe URI.
+   */
+  gst_decodebin3_signals[SIGNAL_ABOUT_TO_FINISH] =
+      g_signal_new ("about-to-finish", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE,
+      0, G_TYPE_NONE);
+
 
   element_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_decodebin3_request_new_pad);
@@ -595,7 +606,7 @@ gst_decodebin3_init (GstDecodebin3 * dbin)
       "max-size-buffers", 0, "use-interleave", TRUE, NULL);
   gst_bin_add ((GstBin *) dbin, dbin->multiqueue);
 
-  dbin->current_group_id = G_MAXUINT32;
+  dbin->current_group_id = GST_GROUP_ID_INVALID;
 
   g_mutex_init (&dbin->factories_lock);
   g_mutex_init (&dbin->selection_lock);
@@ -618,7 +629,7 @@ gst_decodebin3_dispose (GObject * object)
     g_list_free (dbin->decoder_factories);
   if (dbin->decodable_factories)
     g_list_free (dbin->decodable_factories);
-  g_list_free (dbin->requested_selection);
+  g_list_free_full (dbin->requested_selection, g_free);
   g_list_free (dbin->active_selection);
   g_list_free (dbin->to_activate);
   g_list_free (dbin->pending_select_streams);
@@ -700,7 +711,7 @@ set_input_group_id (DecodebinInput * input, guint32 * group_id)
   GstDecodebin3 *dbin = input->dbin;
 
   if (input->group_id != *group_id) {
-    if (input->group_id != G_MAXUINT32)
+    if (input->group_id != GST_GROUP_ID_INVALID)
       GST_WARNING_OBJECT (dbin,
           "Group id changed (%" G_GUINT32_FORMAT " -> %" G_GUINT32_FORMAT
           ") on input %p ", input->group_id, *group_id, input);
@@ -708,7 +719,7 @@ set_input_group_id (DecodebinInput * input, guint32 * group_id)
   }
 
   if (*group_id != dbin->current_group_id) {
-    if (dbin->current_group_id == G_MAXUINT32) {
+    if (dbin->current_group_id == GST_GROUP_ID_INVALID) {
       GST_DEBUG_OBJECT (dbin, "Setting current group id to %" G_GUINT32_FORMAT,
           *group_id);
       dbin->current_group_id = *group_id;
@@ -720,6 +731,30 @@ set_input_group_id (DecodebinInput * input, guint32 * group_id)
   return FALSE;
 }
 
+static void
+parsebin_drained_cb (GstElement * parsebin, DecodebinInput * input)
+{
+  GstDecodebin3 *dbin = input->dbin;
+  gboolean all_drained;
+  GList *tmp;
+
+  GST_WARNING_OBJECT (dbin, "input %p drained", input);
+  input->drained = TRUE;
+
+  all_drained = dbin->main_input->drained;
+  for (tmp = dbin->other_inputs; tmp; tmp = tmp->next) {
+    DecodebinInput *data = (DecodebinInput *) tmp->data;
+
+    all_drained &= data->drained;
+  }
+
+  if (all_drained) {
+    GST_WARNING_OBJECT (dbin, "All inputs drained. Posting about-to-finish");
+    g_signal_emit (dbin, gst_decodebin3_signals[SIGNAL_ABOUT_TO_FINISH], 0,
+        NULL);
+  }
+}
+
 /* Call with INPUT_LOCK taken */
 static gboolean
 ensure_input_parsebin (GstDecodebin3 * dbin, DecodebinInput * input)
@@ -738,6 +773,9 @@ ensure_input_parsebin (GstDecodebin3 * dbin, DecodebinInput * input)
     input->pad_removed_sigid =
         g_signal_connect (input->parsebin, "pad-removed",
         (GCallback) parsebin_pad_removed_cb, input);
+    input->drained_sigid =
+        g_signal_connect (input->parsebin, "drained",
+        (GCallback) parsebin_drained_cb, input);
     g_signal_connect (input->parsebin, "autoplug-continue",
         (GCallback) parsebin_autoplug_continue_cb, dbin);
   }
@@ -787,6 +825,24 @@ fail:
   return GST_PAD_LINK_REFUSED;
 }
 
+/* Drop duration query during _input_pad_unlink */
+static GstPadProbeReturn
+query_duration_drop_probe (GstPad * pad, GstPadProbeInfo * info,
+    DecodebinInput * input)
+{
+  GstPadProbeReturn ret = GST_PAD_PROBE_OK;
+
+  if (GST_IS_QUERY (GST_PAD_PROBE_INFO_DATA (info))) {
+    GstQuery *query = GST_PAD_PROBE_INFO_QUERY (info);
+    if (GST_QUERY_TYPE (query) == GST_QUERY_DURATION) {
+      GST_LOG_OBJECT (pad, "stop forwarding query duration");
+      ret = GST_PAD_PROBE_HANDLED;
+    }
+  }
+
+  return ret;
+}
+
 static void
 gst_decodebin3_input_pad_unlink (GstPad * pad, GstObject * parent)
 {
@@ -806,8 +862,54 @@ gst_decodebin3_input_pad_unlink (GstPad * pad, GstObject * parent)
   }
 
   if (GST_OBJECT_PARENT (GST_OBJECT (input->parsebin)) == GST_OBJECT (dbin)) {
+    GstStreamCollection *collection = NULL;
+    gulong probe_id = gst_pad_add_probe (input->parsebin_sink,
+        GST_PAD_PROBE_TYPE_QUERY_UPSTREAM,
+        (GstPadProbeCallback) query_duration_drop_probe, input, NULL);
+
+    /* Clear stream-collection corresponding to current INPUT and post new
+     * stream-collection message, if needed */
+    if (input->collection) {
+      gst_object_unref (input->collection);
+      input->collection = NULL;
+    }
+
+    SELECTION_LOCK (dbin);
+    collection = get_merged_collection (dbin);
+    if (collection && collection != dbin->collection) {
+      GstMessage *msg;
+      GST_DEBUG_OBJECT (dbin, "Update Stream Collection");
+
+      if (dbin->collection)
+        gst_object_unref (dbin->collection);
+      dbin->collection = collection;
+
+      msg =
+          gst_message_new_stream_collection ((GstObject *) dbin,
+          dbin->collection);
+
+      SELECTION_UNLOCK (dbin);
+      gst_element_post_message (GST_ELEMENT_CAST (dbin), msg);
+      update_requested_selection (dbin);
+    } else
+      SELECTION_UNLOCK (dbin);
+
     gst_bin_remove (GST_BIN (dbin), input->parsebin);
     gst_element_set_state (input->parsebin, GST_STATE_NULL);
+    g_signal_handler_disconnect (input->parsebin, input->pad_removed_sigid);
+    g_signal_handler_disconnect (input->parsebin, input->pad_added_sigid);
+    g_signal_handler_disconnect (input->parsebin, input->drained_sigid);
+    gst_pad_remove_probe (input->parsebin_sink, probe_id);
+    gst_object_unref (input->parsebin);
+    gst_object_unref (input->parsebin_sink);
+
+    input->parsebin = NULL;
+    input->parsebin_sink = NULL;
+
+    if (!input->is_main) {
+      dbin->other_inputs = g_list_remove (dbin->other_inputs, input);
+      free_input_async (dbin, input);
+    }
   }
   INPUT_UNLOCK (dbin);
   return;
@@ -826,6 +928,7 @@ free_input (GstDecodebin3 * dbin, DecodebinInput * input)
   if (input->parsebin) {
     g_signal_handler_disconnect (input->parsebin, input->pad_removed_sigid);
     g_signal_handler_disconnect (input->parsebin, input->pad_added_sigid);
+    g_signal_handler_disconnect (input->parsebin, input->drained_sigid);
     gst_element_set_state (input->parsebin, GST_STATE_NULL);
     gst_object_unref (input->parsebin);
     gst_object_unref (input->parsebin_sink);
@@ -835,6 +938,14 @@ free_input (GstDecodebin3 * dbin, DecodebinInput * input)
   g_free (input);
 }
 
+static void
+free_input_async (GstDecodebin3 * dbin, DecodebinInput * input)
+{
+  GST_LOG_OBJECT (dbin, "pushing input %p on thread pool to free", input);
+  gst_element_call_async (GST_ELEMENT_CAST (dbin),
+      (GstElementCallAsyncFunc) free_input, input, NULL);
+}
+
 /* Call with INPUT_LOCK taken */
 static DecodebinInput *
 create_new_input (GstDecodebin3 * dbin, gboolean main)
@@ -844,7 +955,7 @@ create_new_input (GstDecodebin3 * dbin, gboolean main)
   input = g_new0 (DecodebinInput, 1);
   input->dbin = dbin;
   input->is_main = main;
-  input->group_id = G_MAXUINT32;
+  input->group_id = GST_GROUP_ID_INVALID;
   if (main)
     input->ghost_sink = gst_ghost_pad_new_no_target ("sink", GST_PAD_SINK);
   else {
@@ -874,13 +985,13 @@ gst_decodebin3_request_new_pad (GstElement * element, GstPadTemplate * temp,
 
   /* We are ignoring names for the time being, not sure it makes any sense
    * within the context of decodebin3 ... */
-  INPUT_LOCK (dbin);
   input = create_new_input (dbin, FALSE);
   if (input) {
+    INPUT_LOCK (dbin);
     dbin->other_inputs = g_list_append (dbin->other_inputs, input);
     res = input->ghost_sink;
+    INPUT_UNLOCK (dbin);
   }
-  INPUT_UNLOCK (dbin);
 
   return res;
 }
@@ -923,7 +1034,7 @@ gst_decode_bin_update_factories_list (GstDecodebin3 * dbin)
 }
 
 /* Must be called with appropriate lock if list is a protected variable */
-static gboolean
+static const gchar *
 stream_in_list (GList * list, const gchar * sid)
 {
   GList *tmp;
@@ -936,23 +1047,21 @@ stream_in_list (GList * list, const gchar * sid)
 #endif
 
   for (tmp = list; tmp; tmp = tmp->next) {
-    gchar *osid = (gchar *) tmp->data;
+    const gchar *osid = (gchar *) tmp->data;
     if (!g_strcmp0 (sid, osid))
-      return TRUE;
+      return osid;
   }
 
-  return FALSE;
+  return NULL;
 }
 
 static void
-update_requested_selection (GstDecodebin3 * dbin,
-    GstStreamCollection * collection)
+update_requested_selection (GstDecodebin3 * dbin)
 {
   guint i, nb;
   GList *tmp = NULL;
   GstStreamType used_types = 0;
-
-  nb = gst_stream_collection_get_size (collection);
+  GstStreamCollection *collection;
 
   /* 1. Is there a pending SELECT_STREAMS we can return straight away since
    *  the switch handler will take care of the pending selection */
@@ -963,6 +1072,13 @@ update_requested_selection (GstDecodebin3 * dbin,
     goto beach;
   }
 
+  collection = dbin->collection;
+  if (G_UNLIKELY (collection == NULL)) {
+    GST_DEBUG_OBJECT (dbin, "No current GstStreamCollection");
+    goto beach;
+  }
+  nb = gst_stream_collection_get_size (collection);
+
   /* 2. If not, are we in EXPOSE_ALL_MODE ? If so, match everything */
   GST_FIXME_OBJECT (dbin, "Implement EXPOSE_ALL_MODE");
 
@@ -1013,10 +1129,12 @@ beach:
     if (dbin->requested_selection) {
       GST_FIXME_OBJECT (dbin,
           "Replacing non-NULL requested_selection, what should we do ??");
-      g_list_free (dbin->requested_selection);
+      g_list_free_full (dbin->requested_selection, g_free);
     }
-    dbin->requested_selection = tmp;
+    dbin->requested_selection =
+        g_list_copy_deep (tmp, (GCopyFunc) g_strdup, NULL);
     dbin->selection_updated = TRUE;
+    g_list_free (tmp);
   }
   SELECTION_UNLOCK (dbin);
 }
@@ -1046,7 +1164,7 @@ get_merged_collection (GstDecodebin3 * dbin)
 
   if (!needs_merge) {
     GST_DEBUG_OBJECT (dbin, "No need to merge, returning %p", res);
-    return gst_object_ref (res);
+    return res ? gst_object_ref (res) : NULL;
   }
 
   /* We really need to create a new collection */
@@ -1117,22 +1235,22 @@ find_message_parsebin (GstDecodebin3 * dbin, GstElement * child)
   return input;
 }
 
-static gboolean
+static const gchar *
 stream_in_collection (GstDecodebin3 * dbin, gchar * sid)
 {
   guint i, len;
 
   if (dbin->collection == NULL)
-    return FALSE;
+    return NULL;
   len = gst_stream_collection_get_size (dbin->collection);
   for (i = 0; i < len; i++) {
     GstStream *stream = gst_stream_collection_get_stream (dbin->collection, i);
     const gchar *osid = gst_stream_get_stream_id (stream);
     if (!g_strcmp0 (sid, osid))
-      return TRUE;
+      return osid;
   }
 
-  return FALSE;
+  return NULL;
 }
 
 /* Call with INPUT_LOCK taken */
@@ -1189,6 +1307,7 @@ handle_stream_collection (GstDecodebin3 * dbin,
 #endif
 
   /* Store collection for later usage */
+  SELECTION_LOCK (dbin);
   if (dbin->collection == NULL) {
     dbin->collection = collection;
   } else {
@@ -1204,6 +1323,7 @@ handle_stream_collection (GstDecodebin3 * dbin,
     /* dbin->pending_collection = */
     /*     g_list_append (dbin->pending_collection, collection); */
   }
+  SELECTION_UNLOCK (dbin);
 }
 
 static void
@@ -1226,6 +1346,8 @@ gst_decodebin3_handle_message (GstBin * bin, GstMessage * message)
         posting_collection = TRUE;
         INPUT_UNLOCK (dbin);
       }
+
+      SELECTION_LOCK (dbin);
       if (dbin->collection && collection != dbin->collection) {
         /* Replace collection message, we most likely aggregated it */
         GstMessage *new_msg;
@@ -1235,6 +1357,8 @@ gst_decodebin3_handle_message (GstBin * bin, GstMessage * message)
         gst_message_unref (message);
         message = new_msg;
       }
+      SELECTION_UNLOCK (dbin);
+
       if (collection)
         gst_object_unref (collection);
       break;
@@ -1247,7 +1371,7 @@ gst_decodebin3_handle_message (GstBin * bin, GstMessage * message)
 
   if (posting_collection) {
     /* Figure out a selection for that collection */
-    update_requested_selection (dbin, dbin->collection);
+    update_requested_selection (dbin);
   }
 }
 
@@ -1281,6 +1405,7 @@ get_output_for_slot (MultiQueueSlot * slot)
   DecodebinOutputStream *output = NULL;
   const gchar *stream_id;
   GstCaps *caps;
+  gchar *id_in_list = NULL;
 
   /* If we already have a configured output, just use it */
   if (slot->output != NULL)
@@ -1288,7 +1413,7 @@ get_output_for_slot (MultiQueueSlot * slot)
 
   /*
    * FIXME
-   * 
+   *
    * This method needs to be split into multiple parts
    *
    * 1) Figure out whether stream should be exposed or not
@@ -1331,7 +1456,8 @@ get_output_for_slot (MultiQueueSlot * slot)
 #endif
 
   /* 3. In default mode check if we should expose */
-  if (stream_in_list (dbin->requested_selection, stream_id)) {
+  id_in_list = (gchar *) stream_in_list (dbin->requested_selection, stream_id);
+  if (id_in_list) {
     /* Check if we can steal an existing output stream we could re-use.
      * that is:
      * * an output stream whose slot->stream is not in requested
@@ -1343,7 +1469,8 @@ get_output_for_slot (MultiQueueSlot * slot)
       dbin->to_activate =
           g_list_append (dbin->to_activate, (gchar *) stream_id);
       dbin->requested_selection =
-          g_list_remove (dbin->requested_selection, stream_id);
+          g_list_remove (dbin->requested_selection, id_in_list);
+      g_free (id_in_list);
       SELECTION_UNLOCK (dbin);
       gst_pad_add_probe (output->slot->src_pad, GST_PAD_PROBE_TYPE_IDLE,
           (GstPadProbeCallback) slot_unassign_probe, output->slot, NULL);
@@ -1408,6 +1535,85 @@ is_selection_done (GstDecodebin3 * dbin)
   return msg;
 }
 
+/* Must be called with SELECTION_LOCK taken */
+static void
+check_all_slot_for_eos (GstDecodebin3 * dbin)
+{
+  gboolean all_drained = TRUE;
+  GList *iter;
+
+  GST_DEBUG_OBJECT (dbin, "check slot for eos");
+
+  for (iter = dbin->slots; iter; iter = iter->next) {
+    MultiQueueSlot *slot = iter->data;
+
+    if (!slot->output)
+      continue;
+
+    if (slot->is_drained) {
+      GST_LOG_OBJECT (slot->sink_pad, "slot %p is drained", slot);
+      continue;
+    }
+
+    all_drained = FALSE;
+    break;
+  }
+
+  if (all_drained) {
+    INPUT_LOCK (dbin);
+    if (!pending_pads_are_eos (dbin->main_input))
+      all_drained = FALSE;
+
+    if (all_drained) {
+      for (iter = dbin->other_inputs; iter; iter = iter->next) {
+        if (!pending_pads_are_eos ((DecodebinInput *) iter->data)) {
+          all_drained = FALSE;
+          break;
+        }
+      }
+    }
+    INPUT_UNLOCK (dbin);
+  }
+
+  if (all_drained) {
+    GST_DEBUG_OBJECT (dbin,
+        "All active slots are drained, and no pending input, push EOS");
+
+    for (iter = dbin->input_streams; iter; iter = iter->next) {
+      DecodebinInputStream *input = (DecodebinInputStream *) iter->data;
+      GstPad *peer = gst_pad_get_peer (input->srcpad);
+
+      /* Send EOS to all slots */
+      if (peer) {
+        GstStructure *s;
+        GstEvent *stream_start, *eos;
+
+        stream_start =
+            gst_pad_get_sticky_event (input->srcpad, GST_EVENT_STREAM_START, 0);
+
+        /* First forward a custom STREAM_START event to reset the EOS status (if any) */
+        if (stream_start) {
+          GstStructure *s;
+          GstEvent *custom_stream_start = gst_event_copy (stream_start);
+          gst_event_unref (stream_start);
+          s = (GstStructure *) gst_event_get_structure (custom_stream_start);
+          gst_structure_set (s, "decodebin3-flushing-stream-start",
+              G_TYPE_BOOLEAN, TRUE, NULL);
+          gst_pad_send_event (peer, custom_stream_start);
+        }
+
+        eos = gst_event_new_eos ();
+        s = gst_event_writable_structure (eos);
+        gst_structure_set (s, "decodebin3-custom-final-eos", G_TYPE_BOOLEAN,
+            TRUE, NULL);
+        gst_pad_send_event (peer, eos);
+        gst_object_unref (peer);
+      } else
+        GST_DEBUG_OBJECT (dbin, "no output");
+    }
+  }
+}
+
 static GstPadProbeReturn
 multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
     MultiQueueSlot * slot)
@@ -1423,7 +1629,16 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
       case GST_EVENT_STREAM_START:
       {
         GstStream *stream = NULL;
-        const gchar *stream_id;
+        const GstStructure *s = gst_event_get_structure (ev);
+
+        /* Drop STREAM_START events used to cleanup multiqueue */
+        if (s
+            && gst_structure_has_field (s,
+                "decodebin3-flushing-stream-start")) {
+          ret = GST_PAD_PROBE_HANDLED;
+          gst_event_unref (ev);
+          break;
+        }
 
         gst_event_parse_stream (ev, &stream);
         if (stream == NULL) {
@@ -1431,8 +1646,9 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
               "Got a STREAM_START event without a GstStream");
           break;
         }
-        stream_id = gst_stream_get_stream_id (stream);
-        GST_DEBUG_OBJECT (pad, "Stream Start '%s'", stream_id);
+        slot->is_drained = FALSE;
+        GST_DEBUG_OBJECT (pad, "Stream Start '%s'",
+            gst_stream_get_stream_id (stream));
         if (slot->active_stream == NULL) {
           slot->active_stream = stream;
         } else if (slot->active_stream != stream) {
@@ -1487,7 +1703,37 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
       }
         break;
       case GST_EVENT_EOS:
-        /* FIXME : Figure out */
+      {
+        const GstStructure *s = gst_event_get_structure (ev);
+        gboolean was_drained = slot->is_drained;
+        slot->is_drained = TRUE;
+
+        /* Custom EOS handling first */
+        if (s && gst_structure_has_field (s, "decodebin3-custom-eos")) {
+          GST_LOG_OBJECT (pad, "Received custom EOS");
+          ret = GST_PAD_PROBE_HANDLED;
+          SELECTION_LOCK (dbin);
+          if (slot->input == NULL) {
+            GST_DEBUG_OBJECT (pad,
+                "Got custom-eos from null input stream, remove output stream");
+            /* Remove the output */
+            if (slot->output) {
+              DecodebinOutputStream *output = slot->output;
+              dbin->output_streams =
+                  g_list_remove (dbin->output_streams, output);
+              free_output_stream (dbin, output);
+            }
+            slot->probe_id = 0;
+            dbin->slots = g_list_remove (dbin->slots, slot);
+            free_multiqueue_slot_async (dbin, slot);
+            ret = GST_PAD_PROBE_REMOVE;
+          } else if (!was_drained) {
+            check_all_slot_for_eos (dbin);
+          }
+          SELECTION_UNLOCK (dbin);
+          break;
+        }
+
         GST_FIXME_OBJECT (pad, "EOS on multiqueue source pad. input:%p",
             slot->input);
         if (slot->input == NULL) {
@@ -1510,9 +1756,25 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
             dbin->output_streams = g_list_remove (dbin->output_streams, output);
             free_output_stream (dbin, output);
           }
+          slot->probe_id = 0;
+          dbin->slots = g_list_remove (dbin->slots, slot);
+          SELECTION_UNLOCK (dbin);
+
+          free_multiqueue_slot_async (dbin, slot);
+          ret = GST_PAD_PROBE_REMOVE;
+        } else if (s
+            && gst_structure_has_field (s, "decodebin3-custom-final-eos")) {
+          GST_DEBUG_OBJECT (pad, "Got final eos, propagating downstream");
+        } else {
+          GST_DEBUG_OBJECT (pad, "Got regular eos (all_inputs_are_eos)");
+          /* drop current event as eos will be sent in check_all_slot_for_eos
+           * when all output streams are also eos */
+          ret = GST_PAD_PROBE_DROP;
+          SELECTION_LOCK (dbin);
+          check_all_slot_for_eos (dbin);
           SELECTION_UNLOCK (dbin);
-          ret = GST_PAD_PROBE_HANDLED;
         }
+      }
         break;
       default:
         break;
@@ -1559,11 +1821,14 @@ create_new_slot (GstDecodebin3 * dbin, GstStreamType type)
       gst_stream_type_get_name (type));
   slot = g_new0 (MultiQueueSlot, 1);
   slot->dbin = dbin;
+
   slot->id = dbin->slot_id++;
+
   slot->type = type;
   slot->sink_pad = gst_element_get_request_pad (dbin->multiqueue, "sink_%u");
   if (slot->sink_pad == NULL)
     goto fail;
+
   it = gst_pad_iterate_internal_links (slot->sink_pad);
   if (!it || (gst_iterator_next (it, &item)) != GST_ITERATOR_OK
       || ((slot->src_pad = g_value_dup_object (&item)) == NULL)) {
@@ -1584,7 +1849,9 @@ create_new_slot (GstDecodebin3 * dbin, GstStreamType type)
 
   GST_DEBUG ("Created new slot %u (%p) (%s:%s)", slot->id, slot,
       GST_DEBUG_PAD_NAME (slot->src_pad));
+
   dbin->slots = g_list_append (dbin->slots, slot);
+
   return slot;
 
   /* ERRORS */
@@ -1661,7 +1928,6 @@ get_slot_for_input (GstDecodebin3 * dbin, DecodebinInputStream * input)
 static void
 link_input_to_slot (DecodebinInputStream * input, MultiQueueSlot * slot)
 {
-  GstEvent *event;
   if (slot->input != NULL && slot->input != input) {
     GST_ERROR_OBJECT (slot->dbin,
         "Trying to link input to an already used slot");
@@ -1670,9 +1936,6 @@ link_input_to_slot (DecodebinInputStream * input, MultiQueueSlot * slot)
   gst_pad_link_full (input->srcpad, slot->sink_pad, GST_PAD_LINK_CHECK_NOTHING);
   slot->pending_stream = input->active_stream;
   slot->input = input;
-  event = gst_pad_get_sticky_event (input->srcpad, GST_EVENT_STREAM_START, 0);
-  if (event)
-    gst_pad_send_event (slot->sink_pad, event);
 }
 
 #if 0
@@ -2030,7 +2293,7 @@ reassign_slot (GstDecodebin3 * dbin, MultiQueueSlot * slot)
       tsid = tmp->data;
       /* Pass target stream id to requested selection */
       dbin->requested_selection =
-          g_list_append (dbin->requested_selection, tmp->data);
+          g_list_append (dbin->requested_selection, g_strdup (tmp->data));
       dbin->to_activate = g_list_remove (dbin->to_activate, tmp->data);
       break;
     }
@@ -2159,7 +2422,9 @@ handle_stream_switch (GstDecodebin3 * dbin, GList * select_streams,
       if (slot_to_deactivate) {
         GST_DEBUG_OBJECT (dbin,
             "Slot %p (%s) should be deactivated, no longer used", slot,
-            gst_stream_get_stream_id (slot->active_stream));
+            slot->
+            active_stream ? gst_stream_get_stream_id (slot->active_stream) :
+            "NULL");
         to_deactivate = g_list_append (to_deactivate, slot);
       }
     }
@@ -2228,27 +2493,41 @@ handle_stream_switch (GstDecodebin3 * dbin, GList * select_streams,
   if (to_activate == NULL && pending_streams != NULL) {
     GST_DEBUG_OBJECT (dbin, "Stream switch requested for future collection");
     if (dbin->requested_selection)
-      g_list_free (dbin->requested_selection);
-    dbin->requested_selection = select_streams;
+      g_list_free_full (dbin->requested_selection, g_free);
+    dbin->requested_selection =
+        g_list_copy_deep (select_streams, (GCopyFunc) g_strdup, NULL);
     g_list_free (to_deactivate);
     g_list_free (pending_streams);
     to_deactivate = NULL;
+    pending_streams = NULL;
   } else {
     if (dbin->requested_selection)
-      g_list_free (dbin->requested_selection);
-    dbin->requested_selection = future_request_streams;
+      g_list_free_full (dbin->requested_selection, g_free);
     dbin->requested_selection =
-        g_list_concat (dbin->requested_selection, pending_streams);
+        g_list_copy_deep (future_request_streams, (GCopyFunc) g_strdup, NULL);
+    dbin->requested_selection =
+        g_list_concat (dbin->requested_selection,
+        g_list_copy_deep (pending_streams, (GCopyFunc) g_strdup, NULL));
     if (dbin->to_activate)
       g_list_free (dbin->to_activate);
-    dbin->to_activate = to_reassign;
+    dbin->to_activate = g_list_copy (to_reassign);
   }
 
   dbin->selection_updated = TRUE;
   SELECTION_UNLOCK (dbin);
 
-  if (unknown)
+  if (unknown) {
     GST_FIXME_OBJECT (dbin, "Got request for an unknown stream");
+    g_list_free (unknown);
+  }
+
+  if (to_activate && !slots_to_reassign) {
+    for (tmp = to_activate; tmp; tmp = tmp->next) {
+      MultiQueueSlot *slot = (MultiQueueSlot *) tmp->data;
+      gst_pad_add_probe (slot->src_pad, GST_PAD_PROBE_TYPE_IDLE,
+          (GstPadProbeCallback) idle_reconfigure, slot, NULL);
+    }
+  }
 
   /* For all streams to deactivate, add an idle probe where we will do
    * the unassignment and switch over */
@@ -2258,6 +2537,19 @@ handle_stream_switch (GstDecodebin3 * dbin, GList * select_streams,
         (GstPadProbeCallback) slot_unassign_probe, slot, NULL);
   }
 
+  if (to_deactivate)
+    g_list_free (to_deactivate);
+  if (to_activate)
+    g_list_free (to_activate);
+  if (to_reassign)
+    g_list_free (to_reassign);
+  if (future_request_streams)
+    g_list_free (future_request_streams);
+  if (pending_streams)
+    g_list_free (pending_streams);
+  if (slots_to_reassign)
+    g_list_free (slots_to_reassign);
+
   return ret;
 }
 
@@ -2283,6 +2575,8 @@ ghost_pad_event_probe (GstPad * pad, GstPadProbeInfo * info,
         SELECTION_UNLOCK (dbin);
         GST_DEBUG_OBJECT (pad,
             "Already handled/handling that SELECT_STREAMS event");
+        gst_event_unref (event);
+        ret = GST_PAD_PROBE_HANDLED;
         break;
       }
       dbin->select_streams_seqnum = seqnum;
@@ -2303,8 +2597,10 @@ ghost_pad_event_probe (GstPad * pad, GstPadProbeInfo * info,
         gst_event_unref (event);
       }
       /* Finally handle the switch */
-      if (streams)
+      if (streams) {
         handle_stream_switch (dbin, streams, seqnum);
+        g_list_free_full (streams, g_free);
+      }
       ret = GST_PAD_PROBE_HANDLED;
     }
       break;
@@ -2350,8 +2646,10 @@ gst_decodebin3_send_event (GstElement * element, GstEvent * event)
     }
 #endif
     /* Finally handle the switch */
-    if (streams)
+    if (streams) {
       handle_stream_switch (dbin, streams, seqnum);
+      g_list_free_full (streams, g_free);
+    }
 
     gst_event_unref (event);
     return TRUE;
@@ -2377,6 +2675,14 @@ free_multiqueue_slot (GstDecodebin3 * dbin, MultiQueueSlot * slot)
   g_free (slot);
 }
 
+static void
+free_multiqueue_slot_async (GstDecodebin3 * dbin, MultiQueueSlot * slot)
+{
+  GST_LOG_OBJECT (dbin, "pushing multiqueue slot on thread pool to free");
+  gst_element_call_async (GST_ELEMENT_CAST (dbin),
+      (GstElementCallAsyncFunc) free_multiqueue_slot, slot, NULL);
+}
+
 /* Create a DecodebinOutputStream for a given type
  * Note: It will be empty initially, it needs to be configured
  * afterwards */
@@ -2493,6 +2799,7 @@ gst_decodebin3_change_state (GstElement * element, GstStateChange transition)
       }
       g_list_free (dbin->slots);
       dbin->slots = NULL;
+      dbin->current_group_id = GST_GROUP_ID_INVALID;
       /* Free inputs */
     }
       break;
