@@ -69,6 +69,7 @@ typedef struct
   gboolean eos_sent;            /* when EOS was sent downstream */
   gboolean flushing;            /* set after flush-start and before flush-stop */
   gboolean seen_data;
+  gboolean send_gap_event;
   GstClockTime gap_duration;
 
   GstStreamFlags flags;
@@ -228,12 +229,12 @@ gst_stream_synchronizer_wait (GstStreamSynchronizer * self, GstPad * pad)
       break;
     }
 
-    if (self->send_gap_event) {
+    if (stream->send_gap_event) {
       GstEvent *event;
 
       if (!GST_CLOCK_TIME_IS_VALID (stream->segment.position)) {
         GST_WARNING_OBJECT (pad, "Have no position and can't send GAP event");
-        self->send_gap_event = FALSE;
+        stream->send_gap_event = FALSE;
         continue;
       }
 
@@ -251,7 +252,7 @@ gst_stream_synchronizer_wait (GstStreamSynchronizer * self, GstPad * pad)
       if (!ret) {
         return ret;
       }
-      self->send_gap_event = FALSE;
+      stream->send_gap_event = FALSE;
 
       /* force a check on the loop conditions as we unlocked a
        * few lines above and those variables could have changed */
@@ -373,18 +374,31 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
             ostream->wait = FALSE;
 
             if (ostream->segment.format == GST_FORMAT_TIME) {
-              stop_running_time =
-                  gst_segment_to_running_time (&ostream->segment,
-                  GST_FORMAT_TIME, ostream->segment.stop);
+              if (ostream->segment.rate > 0)
+                stop_running_time =
+                    gst_segment_to_running_time (&ostream->segment,
+                    GST_FORMAT_TIME, ostream->segment.stop);
+              else
+                stop_running_time =
+                    gst_segment_to_running_time (&ostream->segment,
+                    GST_FORMAT_TIME, ostream->segment.start);
+
               position_running_time =
                   gst_segment_to_running_time (&ostream->segment,
                   GST_FORMAT_TIME, ostream->segment.position);
 
               position_running_time =
                   MAX (position_running_time, stop_running_time);
-              position_running_time -=
-                  gst_segment_to_running_time (&ostream->segment,
-                  GST_FORMAT_TIME, ostream->segment.start);
+
+              if (ostream->segment.rate > 0)
+                position_running_time -=
+                    gst_segment_to_running_time (&ostream->segment,
+                    GST_FORMAT_TIME, ostream->segment.start);
+              else
+                position_running_time -=
+                    gst_segment_to_running_time (&ostream->segment,
+                    GST_FORMAT_TIME, ostream->segment.stop);
+
               position_running_time = MAX (0, position_running_time);
 
               position = MAX (position, position_running_time);
@@ -497,9 +511,14 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
           continue;
 
         if (ostream->segment.format == GST_FORMAT_TIME) {
-          start_running_time =
-              gst_segment_to_running_time (&ostream->segment,
-              GST_FORMAT_TIME, ostream->segment.start);
+          if (ostream->segment.rate > 0)
+            start_running_time =
+                gst_segment_to_running_time (&ostream->segment,
+                GST_FORMAT_TIME, ostream->segment.start);
+          else
+            start_running_time =
+                gst_segment_to_running_time (&ostream->segment,
+                GST_FORMAT_TIME, ostream->segment.stop);
 
           new_group_start_time = MAX (new_group_start_time, start_running_time);
         }
@@ -602,7 +621,7 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         g_slist_free (pads);
       } else {
         if (seen_data) {
-          self->send_gap_event = TRUE;
+          stream->send_gap_event = TRUE;
           stream->gap_duration = GST_CLOCK_TIME_NONE;
           stream->wait = TRUE;
           ret = gst_stream_synchronizer_wait (self, srcpad);
@@ -749,7 +768,7 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
 
         ostream->segment.position = new_start;
 
-        self->send_gap_event = TRUE;
+        ostream->send_gap_event = TRUE;
         ostream->gap_duration = new_start - position;
         g_cond_broadcast (&ostream->stream_finish_cond);
       }
@@ -781,6 +800,7 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
   stream->segment_seqnum = G_MAXUINT32;
   stream->group_id = G_MAXUINT;
   stream->seen_data = FALSE;
+  stream->send_gap_event = FALSE;
 
   tmp = g_strdup_printf ("sink_%u", self->current_stream_number);
   stream->sinkpad = gst_pad_new_from_static_template (&sinktemplate, tmp);
@@ -903,7 +923,6 @@ gst_stream_synchronizer_change_state (GstElement * element,
     case GST_STATE_CHANGE_NULL_TO_READY:
       GST_DEBUG_OBJECT (self, "State change NULL->READY");
       self->shutdown = FALSE;
-      self->send_gap_event = FALSE;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_DEBUG_OBJECT (self, "State change READY->PAUSED");
@@ -950,7 +969,7 @@ gst_stream_synchronizer_change_state (GstElement * element,
          * chain () will be blocked on pad_push (), so can't trigger the track
          * which reach EOS to send GAP event. */
         if (stream->is_eos && !stream->eos_sent) {
-          self->send_gap_event = TRUE;
+          stream->send_gap_event = TRUE;
           stream->gap_duration = GST_CLOCK_TIME_NONE;
           g_cond_broadcast (&stream->stream_finish_cond);
         }
@@ -965,7 +984,6 @@ gst_stream_synchronizer_change_state (GstElement * element,
       self->group_start_time = 0;
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
-      self->send_gap_event = FALSE;
       for (l = self->streams; l; l = l->next) {
         GstSyncStream *stream = l->data;
 
@@ -975,6 +993,7 @@ gst_stream_synchronizer_change_state (GstElement * element,
         stream->is_eos = FALSE;
         stream->eos_sent = FALSE;
         stream->flushing = FALSE;
+        stream->send_gap_event = FALSE;
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
@@ -1020,10 +1039,8 @@ gst_stream_synchronizer_class_init (GstStreamSynchronizerClass * klass)
 
   gobject_class->finalize = gst_stream_synchronizer_finalize;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&srctemplate));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (element_class, &srctemplate);
+  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
 
   gst_element_class_set_static_metadata (element_class,
       "Stream Synchronizer", "Generic",
