@@ -20,17 +20,22 @@
 
 /**
  * SECTION:element-videotestsrc
+ * @title: videotestsrc
  *
  * The videotestsrc element is used to produce test video data in a wide variety
  * of formats. The video test data produced can be controlled with the "pattern"
  * property.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * By default the videotestsrc will generate data indefinitely, but if the
+ * #GstBaseSrc:num-buffers property is non-zero it will instead generate a
+ * fixed number of video frames and then send EOS.
+ *
+ * ## Example launch line
  * |[
- * gst-launch -v videotestsrc pattern=snow ! ximagesink
- * ]| Shows random noise in an X window.
- * </refsect2>
+ * gst-launch-1.0 -v videotestsrc pattern=snow ! video/x-raw,width=1280,height=720 ! autovideosink
+ * ]|
+ *  Shows random noise in a video window.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,6 +52,9 @@ GST_DEBUG_CATEGORY_STATIC (video_test_src_debug);
 #define GST_CAT_DEFAULT video_test_src_debug
 
 #define DEFAULT_PATTERN            GST_VIDEO_TEST_SRC_SMPTE
+#define DEFAULT_ANIMATION_MODE     GST_VIDEO_TEST_SRC_FRAMES
+#define DEFAULT_MOTION_TYPE        GST_VIDEO_TEST_SRC_WAVY
+#define DEFAULT_FLIP               FALSE
 #define DEFAULT_TIMESTAMP_OFFSET   0
 #define DEFAULT_IS_LIVE            FALSE
 #define DEFAULT_COLOR_SPEC         GST_VIDEO_TEST_SRC_BT601
@@ -75,15 +83,21 @@ enum
   PROP_FOREGROUND_COLOR,
   PROP_BACKGROUND_COLOR,
   PROP_HORIZONTAL_SPEED,
+  PROP_ANIMATION_MODE,
+  PROP_MOTION_TYPE,
+  PROP_FLIP,
   PROP_LAST
 };
 
 
-#define VTS_VIDEO_CAPS GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ";" \
+#define VTS_VIDEO_CAPS GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) "," \
+  "multiview-mode = { mono, left, right }"                              \
+  ";" \
   "video/x-bayer, format=(string) { bggr, rggb, grbg, gbrg }, "        \
   "width = " GST_VIDEO_SIZE_RANGE ", "                                 \
   "height = " GST_VIDEO_SIZE_RANGE ", "                                \
-  "framerate = " GST_VIDEO_FPS_RANGE
+  "framerate = " GST_VIDEO_FPS_RANGE ", "                              \
+  "multiview-mode = { mono, left, right }"
 
 
 static GstStaticPadTemplate gst_video_test_src_template =
@@ -151,6 +165,8 @@ gst_video_test_src_pattern_get_type (void)
     {GST_VIDEO_TEST_SRC_BAR, "Bar", "bar"},
     {GST_VIDEO_TEST_SRC_PINWHEEL, "Pinwheel", "pinwheel"},
     {GST_VIDEO_TEST_SRC_SPOKES, "Spokes", "spokes"},
+    {GST_VIDEO_TEST_SRC_GRADIENT, "Gradient", "gradient"},
+    {GST_VIDEO_TEST_SRC_COLORS, "Colors", "colors"},
     {0, NULL, NULL}
   };
 
@@ -161,6 +177,56 @@ gst_video_test_src_pattern_get_type (void)
   return video_test_src_pattern_type;
 }
 
+
+/*"animation-mode", which can
+ * take the following values: frames (current behaviour that should stay the
+ * default), running time, wall clock time.
+ */
+
+#define GST_TYPE_VIDEO_TEST_SRC_ANIMATION_MODE (gst_video_test_src_animation_mode_get_type ())
+static GType
+gst_video_test_src_animation_mode_get_type (void)
+{
+  static GType video_test_src_animation_mode = 0;
+  static const GEnumValue animation_modes[] = {
+
+    {GST_VIDEO_TEST_SRC_FRAMES, "frame count", "frames"},
+    {GST_VIDEO_TEST_SRC_WALL_TIME, "wall clock time", "wall-time"},
+    {GST_VIDEO_TEST_SRC_RUNNING_TIME, "running time", "running-time"},
+    {0, NULL, NULL}
+  };
+
+  if (!video_test_src_animation_mode) {
+    video_test_src_animation_mode =
+        g_enum_register_static ("GstVideoTestSrcAnimationMode",
+        animation_modes);
+  }
+  return video_test_src_animation_mode;
+}
+
+
+#define GST_TYPE_VIDEO_TEST_SRC_MOTION_TYPE (gst_video_test_src_motion_type_get_type ())
+static GType
+gst_video_test_src_motion_type_get_type (void)
+{
+  static GType video_test_src_motion_type = 0;
+  static const GEnumValue motion_types[] = {
+    {GST_VIDEO_TEST_SRC_WAVY, "Ball waves back and forth, up and down",
+        "wavy"},
+    {GST_VIDEO_TEST_SRC_SWEEP, "1 revolution per second", "sweep"},
+    {GST_VIDEO_TEST_SRC_HSWEEP, "1/2 revolution per second, then reset to top",
+        "hsweep"},
+    {0, NULL, NULL}
+  };
+
+  if (!video_test_src_motion_type) {
+    video_test_src_motion_type =
+        g_enum_register_static ("GstVideoTestSrcMotionType", motion_types);
+  }
+  return video_test_src_motion_type;
+}
+
+
 static void
 gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
 {
@@ -181,10 +247,29 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
       g_param_spec_enum ("pattern", "Pattern",
           "Type of test pattern to generate", GST_TYPE_VIDEO_TEST_SRC_PATTERN,
           DEFAULT_PATTERN, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_ANIMATION_MODE,
+      g_param_spec_enum ("animation-mode", "Animation mode",
+          "For pattern=ball, which counter defines the position of the ball.",
+          GST_TYPE_VIDEO_TEST_SRC_ANIMATION_MODE, DEFAULT_ANIMATION_MODE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_MOTION_TYPE,
+      g_param_spec_enum ("motion", "Motion",
+          "For pattern=ball, what motion the ball does",
+          GST_TYPE_VIDEO_TEST_SRC_MOTION_TYPE, DEFAULT_MOTION_TYPE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_FLIP,
+      g_param_spec_boolean ("flip", "Flip",
+          "For pattern=ball, invert colors every second.",
+          DEFAULT_FLIP, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (gobject_class, PROP_TIMESTAMP_OFFSET,
       g_param_spec_int64 ("timestamp-offset", "Timestamp offset",
-          "An offset added to timestamps set on buffers (in ns)", G_MININT64,
-          G_MAXINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "An offset added to timestamps set on buffers (in ns)", 0,
+          (G_MAXLONG == G_MAXINT64) ? G_MAXINT64 : (G_MAXLONG * GST_SECOND - 1),
+          0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_IS_LIVE,
       g_param_spec_boolean ("is-live", "Is Live",
           "Whether to act as a live source", DEFAULT_IS_LIVE,
@@ -277,8 +362,8 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
       "Video test source", "Source/Video",
       "Creates a test video stream", "David A. Schleef <ds@schleef.org>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_video_test_src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_video_test_src_template);
 
   gstbasesrc_class->set_caps = gst_video_test_src_setcaps;
   gstbasesrc_class->fixate = gst_video_test_src_src_fixate;
@@ -302,24 +387,97 @@ gst_video_test_src_init (GstVideoTestSrc * src)
   src->foreground_color = DEFAULT_FOREGROUND_COLOR;
   src->background_color = DEFAULT_BACKGROUND_COLOR;
   src->horizontal_speed = DEFAULT_HORIZONTAL_SPEED;
+  src->random_state = 0;
 
   /* we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (src), DEFAULT_IS_LIVE);
+
+  src->animation_mode = DEFAULT_ANIMATION_MODE;
+  src->motion_type = DEFAULT_MOTION_TYPE;
+  src->flip = DEFAULT_FLIP;
+
 }
 
 static GstCaps *
 gst_video_test_src_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
 {
+  GstVideoTestSrc *src = GST_VIDEO_TEST_SRC (bsrc);
   GstStructure *structure;
 
-  caps = gst_caps_make_writable (caps);
+  /* Check if foreground color has alpha, if it is the case,
+   * force color format with an alpha channel downstream */
+  if (src->foreground_color >> 24 != 255) {
+    guint i;
+    GstCaps *alpha_only_caps = gst_caps_new_empty ();
 
+    for (i = 0; i < gst_caps_get_size (caps); i++) {
+      const GstVideoFormatInfo *info;
+      const GValue *formats =
+          gst_structure_get_value (gst_caps_get_structure (caps, i),
+          "format");
+
+      if (GST_VALUE_HOLDS_LIST (formats)) {
+        GValue possible_formats = { 0, };
+        guint list_size = gst_value_list_get_size (formats);
+        guint index;
+
+        g_value_init (&possible_formats, GST_TYPE_LIST);
+        for (index = 0; index < list_size; index++) {
+          const GValue *list_item = gst_value_list_get_value (formats, index);
+          info =
+              gst_video_format_get_info (gst_video_format_from_string
+              (g_value_get_string (list_item)));
+          if (GST_VIDEO_FORMAT_INFO_HAS_ALPHA (info)) {
+            GValue tmp = { 0, };
+
+            gst_value_init_and_copy (&tmp, list_item);
+            gst_value_list_append_value (&possible_formats, &tmp);
+          }
+        }
+
+        if (gst_value_list_get_size (&possible_formats)) {
+          GstStructure *astruct =
+              gst_structure_copy (gst_caps_get_structure (caps, i));
+
+          gst_structure_set_value (astruct, "format", &possible_formats);
+          gst_caps_append_structure (alpha_only_caps, astruct);
+        }
+
+      } else if (G_VALUE_HOLDS_STRING (formats)) {
+        info =
+            gst_video_format_get_info (gst_video_format_from_string
+            (g_value_get_string (formats)));
+
+        if (GST_VIDEO_FORMAT_INFO_HAS_ALPHA (info)) {
+          gst_caps_append_structure (alpha_only_caps,
+              gst_structure_copy (gst_caps_get_structure (caps, i)));
+        }
+      } else {
+        g_assert_not_reached ();
+        GST_WARNING ("Unexpected type for video 'format' field: %s",
+            G_VALUE_TYPE_NAME (formats));
+      }
+    }
+
+    if (gst_caps_is_empty (alpha_only_caps)) {
+      GST_WARNING_OBJECT (src,
+          "Foreground color contains alpha, but downstream can't support alpha.");
+    } else {
+      gst_caps_replace (&caps, alpha_only_caps);
+    }
+  }
+
+  caps = gst_caps_make_writable (caps);
   structure = gst_caps_get_structure (caps, 0);
 
   gst_structure_fixate_field_nearest_int (structure, "width", 320);
   gst_structure_fixate_field_nearest_int (structure, "height", 240);
-  gst_structure_fixate_field_nearest_fraction (structure, "framerate", 30, 1);
+
+  if (gst_structure_has_field (structure, "framerate"))
+    gst_structure_fixate_field_nearest_fraction (structure, "framerate", 30, 1);
+  else
+    gst_structure_set (structure, "framerate", GST_TYPE_FRACTION, 30, 1, NULL);
 
   if (gst_structure_has_field (structure, "pixel-aspect-ratio"))
     gst_structure_fixate_field_nearest_fraction (structure,
@@ -340,6 +498,15 @@ gst_video_test_src_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
     gst_structure_set (structure, "interlace-mode", G_TYPE_STRING,
         "progressive", NULL);
 
+  if (gst_structure_has_field (structure, "multiview-mode"))
+    gst_structure_fixate_field_string (structure, "multiview-mode",
+        gst_video_multiview_mode_to_caps_string
+        (GST_VIDEO_MULTIVIEW_MODE_MONO));
+  else
+    gst_structure_set (structure, "multiview-mode", G_TYPE_STRING,
+        gst_video_multiview_mode_to_caps_string (GST_VIDEO_MULTIVIEW_MODE_MONO),
+        NULL);
+
   caps = GST_BASE_SRC_CLASS (parent_class)->fixate (bsrc, caps);
 
   return caps;
@@ -423,6 +590,12 @@ gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     case GST_VIDEO_TEST_SRC_SPOKES:
       videotestsrc->make_image = gst_video_test_src_spokes;
       break;
+    case GST_VIDEO_TEST_SRC_GRADIENT:
+      videotestsrc->make_image = gst_video_test_src_gradient;
+      break;
+    case GST_VIDEO_TEST_SRC_COLORS:
+      videotestsrc->make_image = gst_video_test_src_colors;
+      break;
     default:
       g_assert_not_reached ();
   }
@@ -488,6 +661,16 @@ gst_video_test_src_set_property (GObject * object, guint prop_id,
       break;
     case PROP_HORIZONTAL_SPEED:
       src->horizontal_speed = g_value_get_int (value);
+      break;
+    case PROP_ANIMATION_MODE:
+      src->animation_mode = g_value_get_enum (value);
+      break;
+    case PROP_MOTION_TYPE:
+      src->motion_type = g_value_get_enum (value);
+      break;
+    case PROP_FLIP:
+      src->flip = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -554,6 +737,15 @@ gst_video_test_src_get_property (GObject * object, guint prop_id,
     case PROP_HORIZONTAL_SPEED:
       g_value_set_int (value, src->horizontal_speed);
       break;
+    case PROP_ANIMATION_MODE:
+      g_value_set_enum (value, src->animation_mode);
+      break;
+    case PROP_MOTION_TYPE:
+      g_value_set_enum (value, src->motion_type);
+      break;
+    case PROP_FLIP:
+      g_value_set_boolean (value, src->flip);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -595,7 +787,7 @@ gst_video_test_src_parse_caps (const GstCaps * caps,
     } else if (g_str_equal (str, "grbg")) {
       *x_inv = 0;
       *y_inv = 1;
-    } else if (g_str_equal (str, "grbg")) {
+    } else if (g_str_equal (str, "gbrg")) {
       *x_inv = 1;
       *y_inv = 0;
     } else
@@ -686,6 +878,8 @@ gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
+  GST_OBJECT_LOCK (videotestsrc);
+
   if (gst_structure_has_name (structure, "video/x-raw")) {
     /* we can use the parsing code */
     if (!gst_video_info_from_caps (&info, caps))
@@ -758,16 +952,20 @@ gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
   videotestsrc->running_time = 0;
   videotestsrc->n_frames = 0;
 
+  GST_OBJECT_UNLOCK (videotestsrc);
+
   return TRUE;
 
   /* ERRORS */
 parse_failed:
   {
+    GST_OBJECT_UNLOCK (videotestsrc);
     GST_DEBUG_OBJECT (bsrc, "failed to parse caps");
     return FALSE;
   }
 unsupported_caps:
   {
+    GST_OBJECT_UNLOCK (videotestsrc);
     GST_DEBUG_OBJECT (bsrc, "unsupported caps: %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
@@ -776,7 +974,7 @@ unsupported_caps:
 static gboolean
 gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
 {
-  gboolean res;
+  gboolean res = FALSE;
   GstVideoTestSrc *src;
 
   src = GST_VIDEO_TEST_SRC (bsrc);
@@ -787,11 +985,34 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
       GstFormat src_fmt, dest_fmt;
       gint64 src_val, dest_val;
 
+      GST_OBJECT_LOCK (src);
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
       res =
           gst_video_info_convert (&src->info, src_fmt, src_val, dest_fmt,
           &dest_val);
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
+      GST_OBJECT_UNLOCK (src);
+      break;
+    }
+    case GST_QUERY_LATENCY:
+    {
+      GST_OBJECT_LOCK (src);
+      if (src->info.fps_n > 0) {
+        GstClockTime latency;
+
+        latency =
+            gst_util_uint64_scale (GST_SECOND, src->info.fps_d,
+            src->info.fps_n);
+        GST_OBJECT_UNLOCK (src);
+        gst_query_set_latency (query,
+            gst_base_src_is_live (GST_BASE_SRC_CAST (src)), latency,
+            GST_CLOCK_TIME_NONE);
+        GST_DEBUG_OBJECT (src, "Reporting latency of %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (latency));
+        res = TRUE;
+      } else {
+        GST_OBJECT_UNLOCK (src);
+      }
       break;
     }
     case GST_QUERY_DURATION:{
@@ -801,16 +1022,22 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
         gst_query_parse_duration (query, &format, NULL);
         switch (format) {
           case GST_FORMAT_TIME:{
-            gint64 dur = gst_util_uint64_scale_int_round (bsrc->num_buffers
+            gint64 dur;
+
+            GST_OBJECT_LOCK (src);
+            dur = gst_util_uint64_scale_int_round (bsrc->num_buffers
                 * GST_SECOND, src->info.fps_d, src->info.fps_n);
             res = TRUE;
             gst_query_set_duration (query, GST_FORMAT_TIME, dur);
+            GST_OBJECT_UNLOCK (src);
             goto done;
           }
           case GST_FORMAT_BYTES:
+            GST_OBJECT_LOCK (src);
             res = TRUE;
             gst_query_set_duration (query, GST_FORMAT_BYTES,
                 bsrc->num_buffers * src->info.size);
+            GST_OBJECT_UNLOCK (src);
             goto done;
           default:
             break;
@@ -832,7 +1059,7 @@ gst_video_test_src_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
 {
   /* for live sources, sync on the timestamp of the buffer */
   if (gst_base_src_is_live (basesrc)) {
-    GstClockTime timestamp = GST_BUFFER_DTS (buffer);
+    GstClockTime timestamp = GST_BUFFER_PTS (buffer);
 
     if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
       /* get duration to calculate end time */
@@ -915,18 +1142,18 @@ gst_video_test_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
   }
 
   GST_LOG_OBJECT (src,
-      "creating buffer from pool for frame %d", (gint) src->n_frames);
+      "creating buffer from pool for frame %" G_GINT64_FORMAT, src->n_frames);
 
   if (!gst_video_frame_map (&frame, &src->info, buffer, GST_MAP_WRITE))
     goto invalid_frame;
 
-  GST_BUFFER_DTS (buffer) =
+  GST_BUFFER_PTS (buffer) =
       src->accum_rtime + src->timestamp_offset + src->running_time;
-  GST_BUFFER_PTS (buffer) = GST_BUFFER_DTS (buffer);
+  GST_BUFFER_DTS (buffer) = GST_CLOCK_TIME_NONE;
 
-  gst_object_sync_values (GST_OBJECT (psrc), GST_BUFFER_DTS (buffer));
+  gst_object_sync_values (GST_OBJECT (psrc), GST_BUFFER_PTS (buffer));
 
-  src->make_image (src, &frame);
+  src->make_image (src, GST_BUFFER_PTS (buffer), &frame);
 
   if ((pal = gst_video_format_get_palette (GST_VIDEO_FRAME_FORMAT (&frame),
               &palsize))) {
@@ -949,8 +1176,8 @@ gst_video_test_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
   }
   GST_BUFFER_OFFSET_END (buffer) = GST_BUFFER_OFFSET (buffer) + 1;
   if (src->info.fps_n) {
-    next_time = gst_util_uint64_scale_int (src->n_frames * GST_SECOND,
-        src->info.fps_d, src->info.fps_n);
+    next_time = gst_util_uint64_scale (src->n_frames,
+        src->info.fps_d * GST_SECOND, src->info.fps_n);
     if (src->reverse) {
       GST_BUFFER_DURATION (buffer) = src->running_time - next_time;
     } else {
@@ -987,12 +1214,14 @@ gst_video_test_src_start (GstBaseSrc * basesrc)
 {
   GstVideoTestSrc *src = GST_VIDEO_TEST_SRC (basesrc);
 
+  GST_OBJECT_LOCK (src);
   src->running_time = 0;
   src->n_frames = 0;
   src->accum_frames = 0;
   src->accum_rtime = 0;
 
   gst_video_info_init (&src->info);
+  GST_OBJECT_UNLOCK (src);
 
   return TRUE;
 }
