@@ -19,6 +19,7 @@
 
 /**
  * SECTION:element-ximagesink
+ * @title: ximagesink
  *
  * XImageSink renders video frames to a drawable (XWindow) on a local or remote
  * display. This element can receive a Window ID from the application through
@@ -26,27 +27,24 @@
  * drawable. If no Window ID was provided by the application, the element will
  * create its own internal window and render into it.
  *
- * <refsect2>
- * <title>Scaling</title>
- * <para>
+ * ## Scaling
+ *
  * As standard XImage rendering to a drawable is not scaled, XImageSink will use
  * reverse caps negotiation to try to get scaled video frames for the drawable.
  * This is accomplished by asking the peer pad if it accepts some different caps
  * which in most cases implies that there is a scaling element in the pipeline,
- * or that an element generating the video frames can generate them with a 
+ * or that an element generating the video frames can generate them with a
  * different geometry. This mechanism is handled during buffer allocations, for
  * each allocation request the video sink will check the drawable geometry, look
  * at the #GstXImageSink:force-aspect-ratio property, calculate the geometry of
  * desired video frames and then check that the peer pad accept those new caps.
  * If it does it will then allocate a buffer in video memory with this new
  * geometry and return it with the new caps.
- * </para>
- * </refsect2>
- * <refsect2>
- * <title>Events</title>
- * <para>
+ *
+ * ## Events
+ *
  * XImageSink creates a thread to handle events coming from the drawable. There
- * are several kind of events that can be grouped in 2 big categories: input 
+ * are several kind of events that can be grouped in 2 big categories: input
  * events and window state related events. Input events will be translated to
  * navigation events and pushed upstream for other elements to react on them.
  * This includes events such as pointer moves, key press/release, clicks etc...
@@ -54,49 +52,48 @@
  * is not flowing (GST_STATE_PAUSED). That means that even when the element is
  * paused, it will receive expose events from the drawable and draw the latest
  * frame with correct borders/aspect-ratio.
- * </para>
- * </refsect2>
- * <refsect2>
- * <title>Pixel aspect ratio</title>
- * <para>
+ *
+ * ## Pixel aspect ratio
+ *
  * When changing state to GST_STATE_READY, XImageSink will open a connection to
  * the display specified in the #GstXImageSink:display property or the default
- * display if nothing specified. Once this connection is open it will inspect 
- * the display configuration including the physical display geometry and 
+ * display if nothing specified. Once this connection is open it will inspect
+ * the display configuration including the physical display geometry and
  * then calculate the pixel aspect ratio. When caps negotiation will occur, the
- * video sink will set the calculated pixel aspect ratio on the caps to make 
+ * video sink will set the calculated pixel aspect ratio on the caps to make
  * sure that incoming video frames will have the correct pixel aspect ratio for
  * this display. Sometimes the calculated pixel aspect ratio can be wrong, it is
  * then possible to enforce a specific pixel aspect ratio using the
  * #GstXImageSink:pixel-aspect-ratio property.
- * </para>
- * </refsect2>
- * <refsect2>
- * <title>Examples</title>
+ *
+ * ## Examples
  * |[
  * gst-launch-1.0 -v videotestsrc ! queue ! ximagesink
- * ]| A pipeline to test reverse negotiation. When the test video signal appears
+ * ]|
+ *  A pipeline to test reverse negotiation. When the test video signal appears
  * you can resize the window and see that scaled buffers of the desired size are
  * going to arrive with a short delay. This illustrates how buffers of desired
  * size are allocated along the way. If you take away the queue, scaling will
  * happen almost immediately.
  * |[
  * gst-launch-1.0 -v videotestsrc ! navigationtest ! videoconvert ! ximagesink
- * ]| A pipeline to test navigation events.
+ * ]|
+ *  A pipeline to test navigation events.
  * While moving the mouse pointer over the test signal you will see a black box
- * following the mouse pointer. If you press the mouse button somewhere on the 
+ * following the mouse pointer. If you press the mouse button somewhere on the
  * video and release it somewhere else a green box will appear where you pressed
  * the button and a red one where you released it. (The navigationtest element
  * is part of gst-plugins-good.)
  * |[
  * gst-launch-1.0 -v videotestsrc ! video/x-raw, pixel-aspect-ratio=(fraction)4/3 ! videoscale ! ximagesink
- * ]| This is faking a 4/3 pixel aspect ratio caps on video frames produced by
+ * ]|
+ *  This is faking a 4/3 pixel aspect ratio caps on video frames produced by
  * videotestsrc, in most cases the pixel aspect ratio of the display will be
- * 1/1. This means that videoscale will have to do the scaling to convert 
+ * 1/1. This means that videoscale will have to do the scaling to convert
  * incoming frames to a size that will match the display pixel aspect ratio
- * (from 320x240 to 320x180 in this case). Note that you might have to escape 
+ * (from 320x240 to 320x180 in this case). Note that you might have to escape
  * some characters for your shell like '\(fraction\)'.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -561,7 +558,7 @@ static void
 gst_x_image_sink_handle_xevents (GstXImageSink * ximagesink)
 {
   XEvent e;
-  guint pointer_x = 0, pointer_y = 0;
+  gint pointer_x = 0, pointer_y = 0;
   gboolean pointer_moved = FALSE;
   gboolean exposed = FALSE, configured = FALSE;
 
@@ -1466,6 +1463,7 @@ gst_x_image_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
   GstXImageSink *ximagesink = GST_X_IMAGE_SINK (bsink);
   GstBufferPool *pool = NULL;
   GstCaps *caps;
+  GstVideoInfo info;
   guint size;
   gboolean need_pool;
 
@@ -1474,26 +1472,23 @@ gst_x_image_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
   if (caps == NULL)
     goto no_caps;
 
+  if (!gst_video_info_from_caps (&info, caps))
+    goto invalid_caps;
+
+  /* the normal size of a frame */
+  size = info.size;
+
   if (need_pool) {
-    GstVideoInfo info;
-
-    if (!gst_video_info_from_caps (&info, caps))
-      goto invalid_caps;
-
     pool = gst_x_image_sink_create_pool (ximagesink, caps, info.size, 0);
 
-    /* the normal size of a frame */
-    size = info.size;
-
     if (pool == NULL)
       goto no_pool;
   }
 
-  if (pool) {
-    /* we need at least 2 buffer because we hold on to the last one */
-    gst_query_add_allocation_pool (query, pool, size, 2, 0);
+  /* we need at least 2 buffer because we hold on to the last one */
+  gst_query_add_allocation_pool (query, pool, size, 2, 0);
+  if (pool)
     gst_object_unref (pool);
-  }
 
   /* we also support various metadata */
   gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
@@ -1641,7 +1636,7 @@ gst_x_image_sink_set_window_handle (GstVideoOverlay * overlay, guintptr id)
   if (xwindow) {
     ximagesink->xwindow = xwindow;
     /* Update the window geometry, possibly generating a reconfigure event. */
-    gst_x_image_sink_xwindow_update_geometry(ximagesink);
+    gst_x_image_sink_xwindow_update_geometry (ximagesink);
   }
 
   g_mutex_unlock (&ximagesink->flow_lock);
