@@ -29,8 +29,6 @@
 # include <valgrind/valgrind.h>
 #endif
 
-#include <unistd.h>
-
 #include <gst/check/gstcheck.h>
 
 #include <gst/video/video.h>
@@ -343,9 +341,11 @@ video_format_is_packed (GstVideoFormat fmt)
     case GST_VIDEO_FORMAT_YVU9:
       return FALSE;
     case GST_VIDEO_FORMAT_IYU1:
+    case GST_VIDEO_FORMAT_IYU2:
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
+    case GST_VIDEO_FORMAT_VYUY:
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -610,6 +610,47 @@ GST_START_TEST (test_video_formats)
 
 GST_END_TEST;
 
+GST_START_TEST (test_video_formats_overflow)
+{
+  GstVideoInfo vinfo;
+
+  gst_video_info_init (&vinfo);
+
+  fail_unless (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, 32768,
+          32767));
+  /* fails due to simplification: we forbid some things that would in theory be fine.
+   * We assume a 128 byte alignment for the width currently
+   * fail_unless (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, 32767, 32768));
+   */
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, 32768,
+          32768));
+
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB,
+          G_MAXINT / 2, G_MAXINT));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, G_MAXINT,
+          G_MAXINT / 2));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB,
+          G_MAXINT / 2, G_MAXINT / 2));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, G_MAXINT,
+          G_MAXINT));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB,
+          G_MAXUINT / 2, G_MAXUINT));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, G_MAXUINT,
+          G_MAXUINT / 2));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB,
+          G_MAXUINT / 2, G_MAXUINT / 2));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, G_MAXUINT,
+          G_MAXUINT));
+
+  fail_unless (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB,
+          1073741824 - 128, 1));
+  fail_if (gst_video_info_set_format (&vinfo, GST_VIDEO_FORMAT_ARGB, 1073741824,
+          1));
+
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_video_formats_rgb)
 {
   GstVideoInfo vinfo;
@@ -682,6 +723,47 @@ GST_START_TEST (test_video_formats_rgba_large_dimension)
 
 GST_END_TEST;
 
+GST_START_TEST (test_guess_framerate)
+{
+  /* Check some obvious exact framerates */
+  gint fps_n, fps_d;
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 24, &fps_n, &fps_d));
+  fail_unless (fps_n == 24 && fps_d == 1);
+
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 30, &fps_n, &fps_d));
+  fail_unless (fps_n == 30 && fps_d == 1);
+
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 25, &fps_n, &fps_d));
+  fail_unless (fps_n == 25 && fps_d == 1);
+
+  /* Some NTSC rates: */
+  fail_unless (gst_video_guess_framerate (GST_SECOND * 1001 / 30000, &fps_n,
+          &fps_d));
+  fail_unless (fps_n == 30000 && fps_d == 1001);
+
+  fail_unless (gst_video_guess_framerate (GST_SECOND * 1001 / 24000, &fps_n,
+          &fps_d));
+  fail_unless (fps_n == 24000 && fps_d == 1001);
+
+  fail_unless (gst_video_guess_framerate (GST_SECOND * 1001 / 60000, &fps_n,
+          &fps_d));
+  fail_unless (fps_n == 60000 && fps_d == 1001);
+
+  /* Check some high FPS, low durations */
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 9000, &fps_n, &fps_d));
+  fail_unless (fps_n == 9000 && fps_d == 1);
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 10000, &fps_n, &fps_d));
+  fail_unless (fps_n == 10000 && fps_d == 1);
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 11000, &fps_n, &fps_d));
+  fail_unless (fps_n == 11000 && fps_d == 1);
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 20000, &fps_n, &fps_d));
+  fail_unless (fps_n == 20000 && fps_d == 1);
+  fail_unless (gst_video_guess_framerate (GST_SECOND / 100000, &fps_n, &fps_d));
+  fail_unless (fps_n == 100000 && fps_d == 1);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_dar_calc)
 {
   guint display_ratio_n, display_ratio_d;
@@ -872,7 +954,7 @@ GST_START_TEST (test_parse_colorimetry)
         _16_235, SMPTE240M, SMPTE240M, SMPTE240M),
     MAKE_COLORIMETRY_TEST ("sRGB", "sRGB", "sRGB",
         _0_255, RGB, SRGB, BT709),
-    MAKE_COLORIMETRY_TEST ("bt2020" , "bt2020", "bt2020",
+    MAKE_COLORIMETRY_TEST ("bt2020", "bt2020", "bt2020",
         _16_235, BT2020, BT2020_12, BT2020),
     MAKE_COLORIMETRY_TEST ("1:4:0:0", "1:4:0:0", NULL,
         _0_255, BT601, UNKNOWN, UNKNOWN),
@@ -2755,10 +2837,12 @@ video_suite (void)
 
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_video_formats);
+  tcase_add_test (tc_chain, test_video_formats_overflow);
   tcase_add_test (tc_chain, test_video_formats_rgb);
   tcase_add_test (tc_chain, test_video_formats_rgba_large_dimension);
   tcase_add_test (tc_chain, test_video_formats_all);
   tcase_add_test (tc_chain, test_video_formats_pack_unpack);
+  tcase_add_test (tc_chain, test_guess_framerate);
   tcase_add_test (tc_chain, test_dar_calc);
   tcase_add_test (tc_chain, test_parse_caps_rgb);
   tcase_add_test (tc_chain, test_parse_caps_multiview);
